@@ -20,8 +20,7 @@
 #include "OutputBuffer.hh"
 
 OutputBuffer::OutputBuffer()
-	: Gsv::Buffer()
-{
+	: Gsv::Buffer() {
 	m_regionBeginMark = Gtk::TextMark::create(true);
 	m_regionEndMark = Gtk::TextMark::create(false);
 	add_mark(m_regionBeginMark, begin());
@@ -39,17 +38,16 @@ OutputBuffer::OutputBuffer()
 	m_regionTag->property_background_rgba() = selColor;
 }
 
-void OutputBuffer::save_region_bounds(bool viewSelected)
-{
+void OutputBuffer::save_region_bounds(bool viewSelected) {
 	Gtk::TextIter start = get_iter_at_mark(get_insert());
 	Gtk::TextIter stop = get_iter_at_mark(get_selection_bound());
-	if(viewSelected){
+	if(viewSelected) {
 		bool entireRegion = false;
-		if(start.get_offset() > stop.get_offset()){
+		if(start.get_offset() > stop.get_offset()) {
 			std::swap(start, stop);
 		}
 		// If nothing or only one word is selected, set region to entire document
-		if(start.get_offset() == stop.get_offset() || !Glib::Regex::create("\\s")->match(get_text(start, stop))){
+		if(start.get_offset() == stop.get_offset() || !Glib::Regex::create("\\s")->match(get_text(start, stop))) {
 			start = begin();
 			stop = end();
 			entireRegion = true;
@@ -57,33 +55,35 @@ void OutputBuffer::save_region_bounds(bool viewSelected)
 		remove_tag(m_regionTag, get_iter_at_mark(m_regionBeginMark), get_iter_at_mark(m_regionEndMark));
 		move_mark(m_regionBeginMark, start);
 		move_mark(m_regionEndMark, stop);
-		if(!entireRegion){
+		if(!entireRegion) {
 			apply_tag(m_regionTag, start, stop);
 		}
 	}
 }
 
-void OutputBuffer::get_region_bounds(Gtk::TextIter& start, Gtk::TextIter& stop)
-{
+void OutputBuffer::get_region_bounds(Gtk::TextIter& start, Gtk::TextIter& stop) {
 	start = get_iter_at_mark(m_regionBeginMark);
 	stop = get_iter_at_mark(m_regionEndMark);
 }
 
-Gtk::TextIter OutputBuffer::replace_range(const Glib::ustring &text, const Gtk::TextIter& start, const Gtk::TextIter& end)
-{
+Gtk::TextIter OutputBuffer::replace_range(const Glib::ustring &text, const Gtk::TextIter& start, const Gtk::TextIter& end) {
 	return insert(erase(start, end), text);
 }
 
-bool OutputBuffer::findReplace(bool backwards, bool replace, bool matchCase, const Glib::ustring& searchstr, const Glib::ustring& replacestr, Gtk::TextView* view)
-{
-	if(searchstr.empty()){
+bool OutputBuffer::findReplace(bool backwards, bool replace, bool matchCase, const Glib::ustring& searchstr, const Glib::ustring& replacestr, Gtk::TextView* view) {
+	if(searchstr.empty()) {
 		return false;
 	}
 	Gtk::TextSearchFlags flags = Gtk::TEXT_SEARCH_VISIBLE_ONLY|Gtk::TEXT_SEARCH_TEXT_ONLY;
 	auto comparator = matchCase ?
-				[](const Glib::ustring& s1, const Glib::ustring& s2){ return s1 == s2; } :
-				[](const Glib::ustring& s1, const Glib::ustring& s2){ return s1.lowercase() == s2.lowercase(); };
-	if(!matchCase){
+	[](const Glib::ustring& s1, const Glib::ustring& s2) {
+		return s1 == s2;
+	}
+:
+	[](const Glib::ustring& s1, const Glib::ustring& s2) {
+		return s1.lowercase() == s2.lowercase();
+	};
+	if(!matchCase) {
 		flags |= Gtk::TEXT_SEARCH_CASE_INSENSITIVE;
 	}
 
@@ -92,67 +92,65 @@ bool OutputBuffer::findReplace(bool backwards, bool replace, bool matchCase, con
 
 	Gtk::TextIter start, end;
 	get_selection_bounds(start, end);
-	if(comparator(get_text(start, end, false), searchstr)){
-		if(replace){
+	if(comparator(get_text(start, end, false), searchstr)) {
+		if(replace) {
 			start = end = insert(erase(start, end), replacestr);
 			start.backward_chars(replacestr.length());
 			select_range(start, end);
 			view->scroll_to(end);
 			return true;
 		}
-		if(backwards){
+		if(backwards) {
 			end.backward_char();
-		}else{
+		} else {
 			start.forward_char();
 		}
 	}
 	Gtk::TextIter matchStart, matchEnd;
-	if(backwards){
+	if(backwards) {
 		if(!end.backward_search(searchstr, flags, matchStart, matchEnd, rstart) &&
-		   !rend.backward_search(searchstr, flags, matchStart, matchEnd, rstart))
-		{
+		        !rend.backward_search(searchstr, flags, matchStart, matchEnd, rstart)) {
 			return false;
 		}
-	}else{
+	} else {
 		if(!start.forward_search(searchstr, flags, matchStart, matchEnd, rend) &&
-		   !rstart.forward_search(searchstr, flags, matchStart, matchEnd, rend))
-		{
+		        !rstart.forward_search(searchstr, flags, matchStart, matchEnd, rend)) {
 			return false;
 		}
 	}
 	// FIXME: backward_search appears to be buggy?
-	matchEnd = matchStart; matchEnd.forward_chars(searchstr.length());
+	matchEnd = matchStart;
+	matchEnd.forward_chars(searchstr.length());
 	select_range(matchStart, matchEnd);
 	view->scroll_to(matchStart);
 	return true;
 }
 
-bool OutputBuffer::replaceAll(const Glib::ustring& searchstr, const Glib::ustring& replacestr, bool matchCase)
-{
+bool OutputBuffer::replaceAll(const Glib::ustring& searchstr, const Glib::ustring& replacestr, bool matchCase) {
 	Gtk::TextIter start, end;
 	get_region_bounds(start, end);
 	int startpos = start.get_offset();
 	int endpos = end.get_offset();
 	Gtk::TextSearchFlags flags = Gtk::TEXT_SEARCH_VISIBLE_ONLY|Gtk::TEXT_SEARCH_TEXT_ONLY;
-	if(!matchCase){
+	if(!matchCase) {
 		flags |= Gtk::TEXT_SEARCH_CASE_INSENSITIVE;
 	}
 	int diff = replacestr.length() - searchstr.length();
 	int count = 0;
 	Gtk::TextIter it = get_iter_at_offset(startpos);
-	while(true){
+	while(true) {
 		Gtk::TextIter matchStart, matchEnd;
-		if(!it.forward_search(searchstr, flags, matchStart, matchEnd) || matchEnd.get_offset() > endpos){
+		if(!it.forward_search(searchstr, flags, matchStart, matchEnd) || matchEnd.get_offset() > endpos) {
 			break;
 		}
 		it = insert(erase(matchStart, matchEnd), replacestr);
 		endpos += diff;
 		++count;
-		while(Gtk::Main::events_pending()){
+		while(Gtk::Main::events_pending()) {
 			Gtk::Main::iteration();
 		}
 	}
-	if(count == 0){
+	if(count == 0) {
 		return false;
 	}
 	return true;
