@@ -25,30 +25,26 @@
 #include <cstring>
 #include <sane/saneopts.h>
 
-void ScannerSane::init()
-{
-	m_thread = Glib::Threads::Thread::create([this]{ run(); });
+void ScannerSane::init() {
+	m_thread = Glib::Threads::Thread::create([this] { run(); });
 }
 
-void ScannerSane::redetect(){
+void ScannerSane::redetect() {
 	m_requestQueue.enqueue({Request::Type::Redetect, 0});
 }
 
-void ScannerSane::scan(const Params& params)
-{
+void ScannerSane::scan(const Params& params) {
 	g_assert(params.depth == 8); // only 8 supported
 	m_requestQueue.enqueue({Request::Type::StartScan, new ScanJob(params)});
 }
 
-void ScannerSane::cancel()
-{
+void ScannerSane::cancel() {
 	m_requestQueue.enqueue({Request::Type::Cancel, 0});
 }
 
-void ScannerSane::close()
-{
+void ScannerSane::close() {
 	m_requestQueue.enqueue({Request::Type::Quit, 0});
-	while(!m_finished){
+	while(!m_finished) {
 		if(Gtk::Main::events_pending())
 			Gtk::Main::iteration();
 	}
@@ -58,15 +54,14 @@ void ScannerSane::close()
 
 /***************************** Thread functions ******************************/
 
-void ScannerSane::run()
-{
+void ScannerSane::run() {
 	// Initialize sane
 	SANE_Int version_code;
 	SANE_Status status = sane_init(&version_code, nullptr);
 	g_debug("sane_init() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		g_critical("Unable to initialize SANE backend: %s", sane_strstatus(status));
-		Utils::runInMainThreadBlocking([this]{ m_signal_initFailed.emit(); });
+		Utils::runInMainThreadBlocking([this] { m_signal_initFailed.emit(); });
 		return;
 	}
 	g_debug("SANE version %d.%d.%d", SANE_VERSION_MAJOR(version_code), SANE_VERSION_MINOR(version_code), SANE_VERSION_BUILD(version_code));
@@ -75,37 +70,37 @@ void ScannerSane::run()
 	doRedetect();
 
 	bool need_redetect = false;
-	while(true){
+	while(true) {
 		// Fetch request if requests are pending or wait for request if idle
-		if(!m_requestQueue.empty() || m_state == State::IDLE){
+		if(!m_requestQueue.empty() || m_state == State::IDLE) {
 			Request request = m_requestQueue.dequeue();
-			if(request.type == Request::Type::Redetect){
+			if(request.type == Request::Type::Redetect) {
 				need_redetect = true;
-			}else if(request.type == Request::Type::StartScan){
+			} else if(request.type == Request::Type::StartScan) {
 				setState(State::OPEN);
 				m_job = request.job;
-			}else if(request.type == Request::Type::Cancel){
+			} else if(request.type == Request::Type::Cancel) {
 				failScan(_("Scan canceled"));
-			}else if(request.type == Request::Type::Quit){
+			} else if(request.type == Request::Type::Quit) {
 				doStop();
 				break;
 			}
 		}
 		// Perform operations according to state
-		if(m_state == State::IDLE){
-			if(need_redetect){
+		if(m_state == State::IDLE) {
+			if(need_redetect) {
 				doRedetect();
 				need_redetect = false;
 			}
-		}else if(m_state == State::OPEN){
+		} else if(m_state == State::OPEN) {
 			doOpen();
-		}else if(m_state == State::SET_OPTIONS){
+		} else if(m_state == State::SET_OPTIONS) {
 			doSetOptions();
-		}else if(m_state == State::START){
+		} else if(m_state == State::START) {
 			doStart();
-		}else if(m_state == State::GET_PARAMETERS){
+		} else if(m_state == State::GET_PARAMETERS) {
 			doGetParameters();
-		}else if(m_state == State::READ){
+		} else if(m_state == State::READ) {
 			doRead();
 		}
 	}
@@ -116,31 +111,28 @@ void ScannerSane::run()
 	m_finished = true;
 }
 
-void ScannerSane::setState(State state)
-{
+void ScannerSane::setState(State state) {
 	m_state = state;
-	Utils::runInMainThreadBlocking([this, state]{ m_signal_scanStateChanged.emit(state); });
+	Utils::runInMainThreadBlocking([this, state] { m_signal_scanStateChanged.emit(state); });
 }
 
-void ScannerSane::failScan(const Glib::ustring& errorString)
-{
+void ScannerSane::failScan(const Glib::ustring& errorString) {
 	doStop();
-	Utils::runInMainThreadBlocking([this, errorString]{ m_signal_scanFailed.emit(errorString); });
+	Utils::runInMainThreadBlocking([this, errorString] { m_signal_scanFailed.emit(errorString); });
 }
 
-void ScannerSane::doRedetect()
-{
+void ScannerSane::doRedetect() {
 	std::vector<Device> devices;
 
 	const SANE_Device** device_list = nullptr;
 	SANE_Status status = sane_get_devices(&device_list, false);
 	g_debug("sane_get_devices() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		g_critical("Unable to get SANE devices: %s", sane_strstatus(status));
-	}else{
-		for(int i = 0; device_list[i] != nullptr; ++i){
+	} else {
+		for(int i = 0; device_list[i] != nullptr; ++i) {
 			g_debug("Device: name=\"%s\" vendor=\"%s\" model=\"%s\" type=\"%s\"",
-					device_list[i]->name, device_list[i]->vendor, device_list[i]->model, device_list[i]->type);
+			        device_list[i]->name, device_list[i]->vendor, device_list[i]->model, device_list[i]->type);
 
 			Device scan_device;
 			scan_device.name = device_list[i]->name;
@@ -157,19 +149,18 @@ void ScannerSane::doRedetect()
 		}
 	}
 
-	Utils::runInMainThreadBlocking([this, devices]{ m_signal_devicesDetected.emit(devices); });
+	Utils::runInMainThreadBlocking([this, devices] { m_signal_devicesDetected.emit(devices); });
 }
 
-void ScannerSane::doOpen()
-{
-	if(m_job->params.device.empty()){
+void ScannerSane::doOpen() {
+	if(m_job->params.device.empty()) {
 		failScan(_("No scanner specified"));
 		return;
 	}
 
 	SANE_Status status = sane_open(m_job->params.device.c_str(), &m_job->handle);
 	g_debug("sane_open(\"%s\") -> %s", m_job->params.device.c_str(), sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		g_critical("Unable to get open device: %s", sane_strstatus(status));
 		m_job->handle = nullptr;
 		failScan(_("Unable to connect to scanner"));
@@ -179,8 +170,7 @@ void ScannerSane::doOpen()
 	setState(State::SET_OPTIONS);
 }
 
-void ScannerSane::doSetOptions()
-{
+void ScannerSane::doSetOptions() {
 	g_debug("set_options");
 
 	int index = 0;
@@ -188,16 +178,16 @@ void ScannerSane::doSetOptions()
 	std::map<std::string, int> options;
 
 	/* Build the option table */
-	while((option = sane_get_option_descriptor(m_job->handle, index)) != nullptr){
+	while((option = sane_get_option_descriptor(m_job->handle, index)) != nullptr) {
 		logOption(index, option);
 		if(
-				/* Ignore groups */
-				option->type != SANE_TYPE_GROUP &&
-				/* Option disabled */
-				(option->cap & SANE_CAP_INACTIVE) == 0 &&
-				/* Some options are unnamed (e.g. Option 0) */
-				option->name != nullptr && std::strlen(option->name) > 0
-				){
+		    /* Ignore groups */
+		    option->type != SANE_TYPE_GROUP &&
+		    /* Option disabled */
+		    (option->cap & SANE_CAP_INACTIVE) == 0 &&
+		    /* Some options are unnamed (e.g. Option 0) */
+		    option->name != nullptr && std::strlen(option->name) > 0
+		) {
 			options.insert({option->name, index});
 		}
 		++index;
@@ -207,7 +197,7 @@ void ScannerSane::doSetOptions()
 
 	/* Pick source */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_SOURCE, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		std::vector<std::string> flatbed_sources = {
 			"Auto",
 			SANE_I18N("Auto"),
@@ -263,7 +253,7 @@ void ScannerSane::doSetOptions()
 
 	/* Scan mode (before resolution as it tends to affect that */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_MODE, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		/* The names of scan modes often used in drivers, as taken from the sane-backends source */
 		std::vector<std::string> color_scan_modes = {
 			SANE_VALUE_SCAN_MODE_COLOR,
@@ -324,7 +314,7 @@ void ScannerSane::doSetOptions()
 
 	/* Disable compression, we will compress after scanning */
 	option = getOptionByName(options, m_job->handle, "compression", index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		std::vector<std::string> disable_compression_names = {
 			SANE_I18N("None"),
 			SANE_I18N("none"),
@@ -338,10 +328,10 @@ void ScannerSane::doSetOptions()
 
 	/* Set resolution and bit depth */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_RESOLUTION, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.dpi, &m_job->params.dpi);
-		else{
+		else {
 			int dpi;
 			setIntOption(m_job->handle, option, index, m_job->params.dpi, &dpi);
 			m_job->params.dpi = dpi;
@@ -353,14 +343,14 @@ void ScannerSane::doSetOptions()
 
 	/* Always use maximum scan area - some scanners default to using partial areas.  This should be patched in sane-backends */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_BR_X, index);
-	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, SANE_UNFIX(option->constraint.range->max), nullptr);
 		else
 			setIntOption(m_job->handle, option, index, option->constraint.range->max, nullptr);
 	}
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_BR_Y, index);
-	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, SANE_UNFIX(option->constraint.range->max), nullptr);
 		else
@@ -369,14 +359,14 @@ void ScannerSane::doSetOptions()
 
 	/* Set page dimensions */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_PAGE_WIDTH, index);
-	if(option != nullptr && m_job->params.page_width > 0.0){
+	if(option != nullptr && m_job->params.page_width > 0.0) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.page_width / 10.0, nullptr);
 		else
 			setIntOption(m_job->handle, option, index, m_job->params.page_width / 10, nullptr);
 	}
 	option = getOptionByName(options, m_job->handle, SANE_NAME_PAGE_HEIGHT, index);
-	if(option != nullptr && m_job->params.page_height > 0.0){
+	if(option != nullptr && m_job->params.page_height > 0.0) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.page_height / 10.0, nullptr);
 		else
@@ -386,34 +376,32 @@ void ScannerSane::doSetOptions()
 	setState(State::START);
 }
 
-void ScannerSane::doStart()
-{
+void ScannerSane::doStart() {
 	SANE_Status status = sane_start(m_job->handle);
 	g_debug("sane_start -> %s", sane_strstatus(status));
-	if(status == SANE_STATUS_GOOD){
+	if(status == SANE_STATUS_GOOD) {
 		setState(State::GET_PARAMETERS);
-	}else{
+	} else {
 		doStop();
 		failScan(_("Unable to start scan"));
 	}
 }
 
-void ScannerSane::doGetParameters()
-{
+void ScannerSane::doGetParameters() {
 	SANE_Status status = sane_get_parameters(m_job->handle, &m_job->parameters);
 	g_debug("sane_get_parameters() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		failScan(_("Error communicating with scanner"));
 		return;
 	}
 
 	g_debug("Parameters: format=%s last_frame=%s bytes_per_line=%d pixels_per_line=%d lines=%d depth=%d",
-			getFrameModeString(m_job->parameters.format).c_str(),
-			m_job->parameters.last_frame ? "SANE_TRUE" : "SANE_FALSE",
-			m_job->parameters.bytes_per_line,
-			m_job->parameters.pixels_per_line,
-			m_job->parameters.lines,
-			m_job->parameters.depth);
+	        getFrameModeString(m_job->parameters.format).c_str(),
+	        m_job->parameters.last_frame ? "SANE_TRUE" : "SANE_FALSE",
+	        m_job->parameters.bytes_per_line,
+	        m_job->parameters.pixels_per_line,
+	        m_job->parameters.lines,
+	        m_job->parameters.depth);
 
 	m_job->height = std::max(1, m_job->parameters.lines);
 	m_job->rowstride = m_job->parameters.pixels_per_line * 3; // Buffer is for a 24 bit RGB image
@@ -427,8 +415,7 @@ void ScannerSane::doGetParameters()
 	setState(State::READ);
 }
 
-void ScannerSane::doRead()
-{
+void ScannerSane::doRead() {
 	/* Read as many bytes as we need to complete a line */
 	int n_to_read = m_job->lineBuffer.size() - m_job->nUsed;
 
@@ -438,14 +425,14 @@ void ScannerSane::doRead()
 	g_debug("sane_read(%d) -> (%s, %d)", n_to_read, sane_strstatus(status), n_read);
 
 	/* Completed read */
-	if(status == SANE_STATUS_EOF){
+	if(status == SANE_STATUS_EOF) {
 		if(m_job->parameters.lines > 0 && m_job->lineCount != m_job->parameters.lines)
 			g_critical("Scan completed with %d lines, expected %d lines", m_job->lineCount, m_job->parameters.lines);
 		if(m_job->nUsed > 0)
 			g_critical("Scan complete with %d bytes of unused data", m_job->nUsed);
 		doCompletePage();
 		return;
-	}else if(status != SANE_STATUS_GOOD){
+	} else if(status != SANE_STATUS_GOOD) {
 		g_critical("Unable to read frame from device: %s", sane_strstatus(status));
 		failScan(_("Error communicating with scanner"));
 		return;
@@ -454,34 +441,34 @@ void ScannerSane::doRead()
 	m_job->nUsed += n_read;
 
 	/* If we completed a line, feed it out */
-	if(m_job->nUsed >= (int)m_job->lineBuffer.size()){
+	if(m_job->nUsed >= (int)m_job->lineBuffer.size()) {
 		g_assert(m_job->nUsed == m_job->parameters.bytes_per_line);
 		++m_job->lineCount;
 
 		// Resize image if necessary
-		if(m_job->lineCount > m_job->height){
+		if(m_job->lineCount > m_job->height) {
 			m_job->height = m_job->lineCount;
 			m_job->imgbuf.resize(m_job->rowstride * m_job->height);
 		}
 
 		// Write data
 		int offset = (m_job->lineCount - 1) * m_job->rowstride;
-		if(m_job->parameters.format == SANE_FRAME_GRAY){
-			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i){
+		if(m_job->parameters.format == SANE_FRAME_GRAY) {
+			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i) {
 				std::memset(&m_job->imgbuf[offset + 3*i], m_job->lineBuffer[i], 3);
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_RGB){
+		} else if(m_job->parameters.format == SANE_FRAME_RGB) {
 			std::memcpy(&m_job->imgbuf[offset], &m_job->lineBuffer[0], m_job->lineBuffer.size());
-		}else if(m_job->parameters.format == SANE_FRAME_RED){
-			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_RED) {
+			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 0] = m_job->lineBuffer[i];
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_GREEN){
-			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_GREEN) {
+			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 1] = m_job->lineBuffer[i];
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_BLUE){
-			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_BLUE) {
+			for(std::size_t i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 2] = m_job->lineBuffer[i];
 			}
 		}
@@ -492,14 +479,13 @@ void ScannerSane::doRead()
 	}
 }
 
-void ScannerSane::doCompletePage()
-{
-	if(!m_job->parameters.last_frame){
+void ScannerSane::doCompletePage() {
+	if(!m_job->parameters.last_frame) {
 		setState(State::GET_PARAMETERS);
 		return;
 	}
 	std::string filename = m_job->params.filename;
-	if(m_job->params.type != ScanType::SINGLE){
+	if(m_job->params.type != ScanType::SINGLE) {
 		std::string base, ext;
 		Utils::get_filename_parts(filename, base, ext);
 		filename = Glib::ustring::compose("%1/%2_%3.%4", base, m_job->pageNumber, ext);
@@ -514,9 +500,9 @@ void ScannerSane::doCompletePage()
 	}
 
 	m_job->imgbuf.clear();
-	Utils::runInMainThreadBlocking([this, filename]{ m_signal_pageAvailable.emit(filename); });
+	Utils::runInMainThreadBlocking([this, filename] { m_signal_pageAvailable.emit(filename); });
 
-	if(m_job->params.type != ScanType::SINGLE){
+	if(m_job->params.type != ScanType::SINGLE) {
 		++m_job->pageNumber;
 		setState(State::GET_PARAMETERS);
 		return;
@@ -525,10 +511,9 @@ void ScannerSane::doCompletePage()
 	doStop();
 }
 
-void ScannerSane::doStop()
-{
-	if(m_job != nullptr){
-		if(m_job->handle != nullptr){
+void ScannerSane::doStop() {
+	if(m_job != nullptr) {
+		if(m_job->handle != nullptr) {
 			g_debug("sane_close()");
 			sane_close(m_job->handle);
 		}
@@ -539,10 +524,9 @@ void ScannerSane::doStop()
 }
 
 /***************************** Sane option stuff *****************************/
-const SANE_Option_Descriptor* ScannerSane::getOptionByName(const std::map<std::string, int>& options, SANE_Handle, const std::string& name, int &index)
-{
+const SANE_Option_Descriptor* ScannerSane::getOptionByName(const std::map<std::string, int>& options, SANE_Handle, const std::string& name, int &index) {
 	std::map<std::string, int>::const_iterator it = options.find(name);
-	if(it != options.end()){
+	if(it != options.end()) {
 		index = it->second;
 		return sane_get_option_descriptor(m_job->handle, index);
 	}
@@ -550,8 +534,7 @@ const SANE_Option_Descriptor* ScannerSane::getOptionByName(const std::map<std::s
 	return nullptr;
 }
 
-bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index)
-{
+bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index) {
 	/* Check if supports automatic option */
 	if((option->cap & SANE_CAP_AUTOMATIC) == 0)
 		return false;
@@ -563,9 +546,8 @@ bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descrip
 	return status == SANE_STATUS_GOOD;
 }
 
-void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, bool value, bool* result)
-{
-	if(option->type != SANE_TYPE_BOOL){
+void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, bool value, bool* result) {
+	if(option->type != SANE_TYPE_BOOL) {
 		return;
 	}
 
@@ -575,26 +557,25 @@ void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor
 	g_debug("sane_control_option(%d, SANE_ACTION_SET_VALUE, %s) -> (%s, %s)", option_index, value ? "SANE_TRUE" : "SANE_FALSE", sane_strstatus(status), result ? "SANE_TRUE" : "SANE_FALSE");
 }
 
-void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, int value, int* result)
-{
-	if(option->type != SANE_TYPE_INT){
+void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, int value, int* result) {
+	if(option->type != SANE_TYPE_INT) {
 		return;
 	}
 
 	SANE_Word v = static_cast<SANE_Word>(value);
-	if(option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->constraint.range->quant != 0)
 			v *= option->constraint.range->quant;
 		v = std::max(option->constraint.range->min, std::min(option->constraint.range->max, v));
-	}else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST){
+	} else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
 		int distance = std::numeric_limits<int>::max();
 		int nearest = 0;
 
 		/* Find nearest value to requested */
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			int x = option->constraint.word_list[i+1];
 			int d = std::abs(x - v);
-			if(d < distance){
+			if(d < distance) {
 				distance = d;
 				nearest = x;
 			}
@@ -607,26 +588,25 @@ void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor*
 	if(result) *result = v;
 }
 
-void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, double value, double* result)
-{
-	if(option->type != SANE_TYPE_FIXED){
+void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, double value, double* result) {
+	if(option->type != SANE_TYPE_FIXED) {
 		return;
 	}
 
 	double v = value;
-	if(option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		double min = SANE_UNFIX(option->constraint.range->min);
 		double max = SANE_UNFIX(option->constraint.range->max);
 		v = std::max(min, std::min(max, v));
-	}else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST){
+	} else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
 		double distance = std::numeric_limits<double>::max();
 		double nearest = 0.0;
 
 		/* Find nearest value to requested */
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			double x = SANE_UNFIX(option->constraint.word_list[i+1]);
 			double d = std::abs(x - v);
-			if(d < distance){
+			if(d < distance) {
 				distance = d;
 				nearest = x;
 			}
@@ -641,9 +621,8 @@ void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descripto
 	if(result) *result = SANE_UNFIX(v_fixed);
 }
 
-bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const std::string& value, std::string* result)
-{
-	if(option->type != SANE_TYPE_STRING){
+bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const std::string& value, std::string* result) {
+	if(option->type != SANE_TYPE_STRING) {
 		return false;
 	}
 
@@ -656,18 +635,17 @@ bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descript
 	return status == SANE_STATUS_GOOD;
 }
 
-bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const std::vector<std::string>& values, std::string* result)
-{
-	if(option->type != SANE_TYPE_STRING){
+bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const std::vector<std::string>& values, std::string* result) {
+	if(option->type != SANE_TYPE_STRING) {
 		return false;
 	}
-	if(option->constraint_type != SANE_CONSTRAINT_STRING_LIST){
+	if(option->constraint_type != SANE_CONSTRAINT_STRING_LIST) {
 		return false;
 	}
 
-	for(const std::string& value : values){
-		for(int j = 0; option->constraint.string_list[j] != nullptr; ++j){
-			if(value == option->constraint.string_list[j]){
+	for(const std::string& value : values) {
+		for(int j = 0; option->constraint.string_list[j] != nullptr; ++j) {
+			if(value == option->constraint.string_list[j]) {
 				return setStringOption(handle, option, option_index, value, result);
 			}
 		}
@@ -676,8 +654,7 @@ bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Opti
 	return false;
 }
 
-void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option)
-{
+void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option) {
 	Glib::ustring s = Glib::ustring::compose("Option %1:", index);
 
 	if(option->name && std::strlen(option->name) > 0)
@@ -687,22 +664,22 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		s += Glib::ustring::compose(" title='%1'", option->title);
 
 	Glib::ustring typestr[] = {"bool", "int", "fixed", "string", "button", "group"};
-	if(option->type <= SANE_TYPE_GROUP){
+	if(option->type <= SANE_TYPE_GROUP) {
 		s += Glib::ustring::compose(" type=%1", typestr[option->type]);
-	}else{
+	} else {
 		s += Glib::ustring::compose(" type=%1", option->type);
 	}
 
 	s += Glib::ustring::compose(" size=%1", option->size);
 
 	Glib::ustring unitstr[] = {"none", "pixel", "bit", "mm", "dpi", "percent", "microseconds"};
-	if(option->unit <= SANE_UNIT_MICROSECOND){
+	if(option->unit <= SANE_UNIT_MICROSECOND) {
 		s += Glib::ustring::compose(" unit=%1", unitstr[option->unit]);
-	}else{
+	} else {
 		s += Glib::ustring::compose(" unit=%1", option->unit);
 	}
 
-	switch(option->constraint_type){
+	switch(option->constraint_type) {
 	case SANE_CONSTRAINT_RANGE:
 		if(option->type == SANE_TYPE_FIXED)
 			s += Glib::ustring::compose(" min=%1, max=%2, quant=%3", SANE_UNFIX(option->constraint.range->min), SANE_UNFIX(option->constraint.range->max), option->constraint.range->quant);
@@ -711,7 +688,7 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		break;
 	case SANE_CONSTRAINT_WORD_LIST:
 		s += " values=[";
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			if(i > 0)
 				s += ", ";
 			if(option->type == SANE_TYPE_INT)
@@ -723,7 +700,7 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		break;
 	case SANE_CONSTRAINT_STRING_LIST:
 		s += " values=[";
-		for(int i = 0; option->constraint.string_list[i] != nullptr; ++i){
+		for(int i = 0; option->constraint.string_list[i] != nullptr; ++i) {
 			if(i > 0)
 				s += ", ";
 			s += Glib::ustring::compose("\"%1\"", option->constraint.string_list[i]);
@@ -735,38 +712,38 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 	}
 
 	int cap = option->cap;
-	if(cap != 0){
+	if(cap != 0) {
 		s += " cap=";
-		if((cap & SANE_CAP_SOFT_SELECT) != 0){
+		if((cap & SANE_CAP_SOFT_SELECT) != 0) {
 			s += "soft-select,";
 			cap &= ~SANE_CAP_SOFT_SELECT;
 		}
-		if((cap & SANE_CAP_HARD_SELECT) != 0){
+		if((cap & SANE_CAP_HARD_SELECT) != 0) {
 			s += "hard-select,";
 			cap &= ~SANE_CAP_HARD_SELECT;
 		}
-		if((cap & SANE_CAP_SOFT_DETECT) != 0){
+		if((cap & SANE_CAP_SOFT_DETECT) != 0) {
 			s += "soft-detect,";
 			cap &= ~SANE_CAP_SOFT_DETECT;
 		}
-		if((cap & SANE_CAP_EMULATED) != 0){
+		if((cap & SANE_CAP_EMULATED) != 0) {
 			s += "emulated,";
 			cap &= ~SANE_CAP_EMULATED;
 		}
-		if((cap & SANE_CAP_AUTOMATIC) != 0){
+		if((cap & SANE_CAP_AUTOMATIC) != 0) {
 			s += "automatic,";
 			cap &= ~SANE_CAP_AUTOMATIC;
 		}
-		if((cap & SANE_CAP_INACTIVE) != 0){
+		if((cap & SANE_CAP_INACTIVE) != 0) {
 			s += "inactive,";
 			cap &= ~SANE_CAP_INACTIVE;
 		}
-		if((cap & SANE_CAP_ADVANCED) != 0){
+		if((cap & SANE_CAP_ADVANCED) != 0) {
 			s += "advanced,";
 			cap &= ~SANE_CAP_ADVANCED;
 		}
 		/* Unknown capabilities */
-		if(cap != 0){
+		if(cap != 0) {
 			s += Glib::ustring::compose("unknown (%1)", cap);
 		}
 	}
@@ -776,12 +753,11 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		g_debug("  Description: %s", option->desc);
 }
 
-Glib::ustring ScannerSane::getFrameModeString(SANE_Frame frame)
-{
+Glib::ustring ScannerSane::getFrameModeString(SANE_Frame frame) {
 	Glib::ustring framestr[] = {"SANE_FRAME_GRAY", "SANE_FRAME_RGB", "SANE_FRAME_RED", "SANE_FRAME_GREEN", "SANE_FRAME_BLUE"};
-	if(frame <= SANE_FRAME_BLUE){
+	if(frame <= SANE_FRAME_BLUE) {
 		return framestr[frame];
-	}else{
+	} else {
 		return Glib::ustring::compose("SANE_FRAME(%1)", frame);
 	}
 }
