@@ -20,7 +20,14 @@
 #include "Config.hh"
 #include "MainWindow.hh"
 
+#include <QDesktopServices>
+#include <QDir>
 #include <QMultiMap>
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#include <QStandardPaths>
+#endif
+#include <QUrl>
+#include <tesseract/baseapi.h>
 
 const QList<Config::Lang> Config::LANGUAGES = {
 	// {ISO 639-2, ISO 639-1, name}
@@ -277,8 +284,7 @@ const QMultiMap<QString,QString> Config::LANGUAGE_CULTURES = Config::buildLangua
 
 
 Config::Config(QWidget* parent)
-	: QDialog(parent)
-{
+	: QDialog(parent) {
 	ui.setupUi(this);
 
 #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
@@ -294,7 +300,7 @@ Config::Config(QWidget* parent)
 	ui.checkBoxUpdateCheck->setVisible(false);
 #endif
 
-	for(const Lang& lang : LANGUAGES){
+	for(const Lang& lang : LANGUAGES) {
 		int row = ui.tableWidgetPredefLang->rowCount();
 		ui.tableWidgetPredefLang->insertRow(row);
 		ui.tableWidgetPredefLang->setItem(row, 0, new QTableWidgetItem(lang.prefix));
@@ -314,6 +320,7 @@ Config::Config(QWidget* parent)
 	connect(ui.lineEditLangPrefix, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditErrorState()));
 	connect(ui.lineEditLangName, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditErrorState()));
 	connect(ui.lineEditLangCode, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditErrorState()));
+	connect(ui.comboBoxDataLocation, SIGNAL(currentIndexChanged(int)), this, SLOT(setDataLocations(int)));
 
 	addSetting(new SwitchSetting("dictinstall", ui.checkBoxDictInstall, true));
 	addSetting(new SwitchSetting("updatecheck", ui.checkBoxUpdateCheck, true));
@@ -321,19 +328,18 @@ Config::Config(QWidget* parent)
 	addSetting(new SwitchSetting("systemoutputfont", ui.checkBoxDefaultOutputFont, true));
 	addSetting(new FontSetting("customoutputfont", &m_fontDialog, QFont().toString()));
 	addSetting(new ComboSetting("textencoding", ui.comboBoxEncoding, 0));
+	addSetting(new ComboSetting("datadirs", ui.comboBoxDataLocation, 0));
 
 	updateFontButton(m_fontDialog.currentFont());
 }
 
-Config::~Config()
-{
+Config::~Config() {
 	qDeleteAll(m_settings.values());
 }
 
-bool Config::searchLangSpec(Lang& lang) const
-{
-	for(const QTableWidget* table : QList<QTableWidget*>{ui.tableWidgetPredefLang, ui.tableWidgetAdditionalLang}){
-		for(int row = 0, nRows = table->rowCount(); row < nRows; ++row){
+bool Config::searchLangSpec(Lang& lang) const {
+	for(const QTableWidget* table : QList<QTableWidget*> {ui.tableWidgetPredefLang, ui.tableWidgetAdditionalLang}) {
+		for(int row = 0, nRows = table->rowCount(); row < nRows; ++row) {
 			if(table->item(row, 0)->text() == lang.prefix) {
 				lang = {table->item(row, 0)->text(), table->item(row, 1)->text(), table->item(row, 2)->text()};
 				return true;
@@ -343,41 +349,151 @@ bool Config::searchLangSpec(Lang& lang) const
 	return false;
 }
 
-QList<QString> Config::searchLangCultures(const QString& code) const
-{
+QList<QString> Config::searchLangCultures(const QString& code) const {
 	return LANGUAGE_CULTURES.values(code);
 }
 
-void Config::showDialog()
-{
+void Config::showDialog() {
 	toggleAddLanguage(true);
 	exec();
 	getSetting<TableSetting>("customlangs")->serialize();
 }
 
-bool Config::useUtf8() const
-{
+bool Config::useUtf8() const {
 	return ui.comboBoxEncoding->currentIndex() == 1;
 }
 
-void Config::disableDictInstall()
-{
+bool Config::useSystemDataLocations() const {
+	return ui.comboBoxDataLocation->currentIndex() == 0;
+}
+
+QString Config::tessdataLocation() const {
+	return ui.lineEditTessdataLocation->text();
+}
+
+QString Config::spellingLocation() const {
+	return ui.lineEditSpellLocation->text();
+}
+
+void Config::disableDictInstall() {
 	getSetting<SwitchSetting>("dictinstall")->setValue(false);
 }
 
-void Config::disableUpdateCheck()
-{
+void Config::disableUpdateCheck() {
 	getSetting<SwitchSetting>("updatecheck")->setValue(false);
 }
 
-void Config::toggleAddLanguage(bool forceHide)
-{
+void Config::setDataLocations(int idx) {
+	if(idx == 0) {
+#ifdef Q_OS_WIN
+		QDir dataDir = QDir(QString("%1/../share/").arg(QApplication::applicationDirPath()));
+		qputenv("TESSDATA_PREFIX", dataDir.absolutePath().toLocal8Bit());
+		ui.lineEditSpellLocation->setText(dataDir.absoluteFilePath("myspell/dicts"));
+#else
+		QDir dataDir("/usr/share");
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+		unsetenv("TESSDATA_PREFIX");
+#else
+		qunsetenv("TESSDATA_PREFIX");
+#endif
+		if(QDir(dataDir.absoluteFilePath("myspell/dicts")).exists()) {
+			ui.lineEditSpellLocation->setText(dataDir.absoluteFilePath("myspell/dicts"));
+		} else {
+			ui.lineEditSpellLocation->setText(dataDir.absoluteFilePath("myspell"));
+		}
+#endif
+	} else {
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+# ifdef Q_OS_WIN
+		QDir configDir = QDir(QDir::home().absoluteFilePath("Local Settings/Application Data"));
+# else
+		QDir configDir = QDir(QDir::home().absoluteFilePath(".config"));
+# endif
+#else
+		QDir configDir = QDir(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation));
+#endif
+		qputenv("TESSDATA_PREFIX", configDir.absoluteFilePath("tessdata").toLocal8Bit());
+		ui.lineEditSpellLocation->setText(configDir.absoluteFilePath("enchant/myspell"));
+	}
+	tesseract::TessBaseAPI tess;
+	QByteArray current = setlocale(LC_NUMERIC, NULL);
+	setlocale(LC_NUMERIC, "C");
+	tess.Init(nullptr, nullptr);
+	setlocale(LC_NUMERIC, current.constData());
+	ui.lineEditTessdataLocation->setText(QString(tess.GetDatapath()));
+}
+
+void Config::openTessdataDir() {
+	int idx = QSettings().value("datadirs").toInt();
+	if(idx == 0) {
+#ifdef Q_OS_WIN
+		QDir dataDir = QDir(QString("%1/../share/").arg(QApplication::applicationDirPath()));
+		qputenv("TESSDATA_PREFIX", dataDir.absolutePath().toLocal8Bit());
+#else
+# if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+		unsetenv("TESSDATA_PREFIX");
+# else
+		qunsetenv("TESSDATA_PREFIX");
+# endif
+#endif
+	} else {
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+# ifdef Q_OS_WIN
+		QDir configDir = QDir(QDir::home().absoluteFilePath("Local Settings/Application Data"));
+# else
+		QDir configDir = QDir(QDir::home().absoluteFilePath(".config"));
+# endif
+#else
+		QDir configDir = QDir(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation));
+#endif
+		qputenv("TESSDATA_PREFIX", configDir.absoluteFilePath("tessdata").toLocal8Bit());
+	}
+	tesseract::TessBaseAPI tess;
+	QByteArray current = setlocale(LC_NUMERIC, NULL);
+	setlocale(LC_NUMERIC, "C");
+	tess.Init(nullptr, nullptr);
+	setlocale(LC_NUMERIC, current.constData());
+	QDir().mkpath(QString(tess.GetDatapath()));
+	QDesktopServices::openUrl(QUrl::fromLocalFile(QString(tess.GetDatapath())));
+}
+
+void Config::openSpellingDir() {
+	int idx = QSettings().value("datadirs").toInt();
+	if(idx == 0) {
+#ifdef Q_OS_WIN
+		QDir dataDir = QDir(QString("%1/../share/").arg(QApplication::applicationDirPath()));
+		QDir().mkpath(dataDir.absoluteFilePath("myspell/dicts"));
+		QDesktopServices::openUrl(QUrl::fromLocalFile(dataDir.absoluteFilePath("myspell/dicts")));
+#else
+		QDir dataDir("/usr/share");
+		if(QDir(dataDir.absoluteFilePath("myspell/dicts")).exists()) {
+			QDesktopServices::openUrl(QUrl::fromLocalFile(dataDir.absoluteFilePath("myspell/dicts")));
+		} else {
+			QDesktopServices::openUrl(QUrl::fromLocalFile(dataDir.absoluteFilePath("myspell")));
+		}
+#endif
+	} else {
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+# ifdef Q_OS_WIN
+		QDir configDir = QDir(QDir::home().absoluteFilePath("Local Settings/Application Data"));
+# else
+		QDir configDir = QDir(QDir::home().absoluteFilePath(".config"));
+# endif
+#else
+		QDir configDir = QDir(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation));
+#endif
+		QDir().mkpath(configDir.absoluteFilePath("enchant/myspell"));
+		QDesktopServices::openUrl(QUrl::fromLocalFile(configDir.absoluteFilePath("enchant/myspell")));
+	}
+}
+
+void Config::toggleAddLanguage(bool forceHide) {
 	bool addVisible = forceHide ? true : ui.widgetAddLang->isVisible();
 	ui.widgetAddLang->setVisible(!addVisible);
 	ui.widgetAddRemoveLang->setVisible(addVisible);
-	if(addVisible){
+	if(addVisible) {
 		ui.pushButtonAddLang->setFocus();
-	}else{
+	} else {
 		ui.pushButtonAddLangOk->setFocus();
 	}
 	ui.lineEditLangPrefix->setText("");
@@ -388,25 +504,23 @@ void Config::toggleAddLanguage(bool forceHide)
 	ui.lineEditLangName->setStyleSheet("");
 }
 
-void Config::addLanguage()
-{
+void Config::addLanguage() {
 	QString errorStyle = "background: #FF7777; color: #FFFFFF;";
 	bool invalid = false;
-	if(QRegExp("^\\w+$").indexIn(ui.lineEditLangPrefix->text()) == -1){
+	if(QRegExp("^\\w+$").indexIn(ui.lineEditLangPrefix->text()) == -1) {
 		invalid = true;
 		ui.lineEditLangPrefix->setStyleSheet(errorStyle);
 	}
-	if(QRegExp("^.+$").indexIn(ui.lineEditLangName->text()) == -1){
+	if(QRegExp("^.+$").indexIn(ui.lineEditLangName->text()) == -1) {
 		invalid = true;
 		ui.lineEditLangName->setStyleSheet(errorStyle);
 	}
 	if(QRegExp("^[a-z]{2}$").indexIn(ui.lineEditLangCode->text()) == -1 &&
-	   QRegExp("^[a-z]{2}_[A-Z]{2}$").indexIn(ui.lineEditLangCode->text()) == -1)
-	{
+	        QRegExp("^[a-z]{2}_[A-Z]{2}$").indexIn(ui.lineEditLangCode->text()) == -1) {
 		invalid = true;
 		ui.lineEditLangCode->setStyleSheet(errorStyle);
 	}
-	if(!invalid){
+	if(!invalid) {
 		int row = ui.tableWidgetAdditionalLang->rowCount();
 		ui.tableWidgetAdditionalLang->insertRow(row);
 		ui.tableWidgetAdditionalLang->setItem(row, 0, new QTableWidgetItem(ui.lineEditLangPrefix->text()));
@@ -419,25 +533,20 @@ void Config::addLanguage()
 	}
 }
 
-void Config::removeLanguage()
-{
-	for(const QModelIndex& index : ui.tableWidgetAdditionalLang->selectionModel()->selectedRows())
-	{
+void Config::removeLanguage() {
+	for(const QModelIndex& index : ui.tableWidgetAdditionalLang->selectionModel()->selectedRows()) {
 		ui.tableWidgetAdditionalLang->removeRow(index.row());
 	}
 }
 
-void Config::updateFontButton(const QFont &font)
-{
+void Config::updateFontButton(const QFont &font) {
 	ui.pushButtonOutputFont->setText(QString("%1 %2").arg(font.family()).arg(font.pointSize()));
 }
 
-void Config::langTableSelectionChanged(const QItemSelection &selected, const QItemSelection &/*deselected*/)
-{
+void Config::langTableSelectionChanged(const QItemSelection &selected, const QItemSelection &/*deselected*/) {
 	ui.pushButtonRemoveLang->setDisabled(selected.isEmpty());
 }
 
-void Config::clearLineEditErrorState()
-{
+void Config::clearLineEditErrorState() {
 	static_cast<QLineEdit*>(QObject::sender())->setStyleSheet("");
 }
