@@ -25,14 +25,12 @@
 #include <QSyntaxHighlighter>
 
 
-class OutputTextEdit::WhitespaceHighlighter : public QSyntaxHighlighter
-{
+class OutputTextEdit::WhitespaceHighlighter : public QSyntaxHighlighter {
 public:
 	WhitespaceHighlighter(QTextDocument* document)
 		: QSyntaxHighlighter(document) {}
 private:
-	void highlightBlock(const QString &text)
-	{
+	void highlightBlock(const QString &text) {
 		QTextCharFormat fmt;
 		fmt.setForeground(Qt::gray);
 
@@ -48,8 +46,7 @@ private:
 
 
 OutputTextEdit::OutputTextEdit(QWidget *parent)
-	: QPlainTextEdit(parent)
-{
+	: QPlainTextEdit(parent) {
 	m_wsHighlighter = new WhitespaceHighlighter(document());
 
 	m_regionCursor = textCursor();
@@ -63,19 +60,20 @@ OutputTextEdit::OutputTextEdit(QWidget *parent)
 	// Force inactive selection to have same color as active selection
 	QColor highlightColor = palette().color(QPalette::Highlight);
 	QColor highlightedTextColor = palette().color(QPalette::HighlightedText);
-	auto colorToString = [&](const QColor& color){ return QString("rgb(%1, %2, %3)").arg(color.red()).arg(color.green()).arg(color.blue()); };
+	auto colorToString = [&](const QColor& color) {
+		return QString("rgb(%1, %2, %3)").arg(color.red()).arg(color.green()).arg(color.blue());
+	};
 	setStyleSheet(QString("QPlainTextEdit { selection-background-color: %1; selection-color: %2; }")
-				  .arg(colorToString(highlightColor), colorToString(highlightedTextColor)));
+	              .arg(colorToString(highlightColor), colorToString(highlightedTextColor)));
 }
 
-OutputTextEdit::~OutputTextEdit()
-{
+OutputTextEdit::~OutputTextEdit() {
 	delete m_wsHighlighter;
 }
 
-QTextCursor OutputTextEdit::regionBounds() const{
+QTextCursor OutputTextEdit::regionBounds() const {
 	QTextCursor c = m_regionCursor;
-	if(c.anchor() > c.position()){
+	if(c.anchor() > c.position()) {
 		int pos = c.anchor();
 		int anchor = c.position();
 		c.setPosition(anchor);
@@ -84,19 +82,18 @@ QTextCursor OutputTextEdit::regionBounds() const{
 	return c;
 }
 
-bool OutputTextEdit::findReplace(bool backwards, bool replace, bool matchCase, const QString &searchstr, const QString &replacestr)
-{
+bool OutputTextEdit::findReplace(bool backwards, bool replace, bool matchCase, const QString &searchstr, const QString &replacestr) {
 	clearFocus();
-	if(searchstr.isEmpty()){
+	if(searchstr.isEmpty()) {
 		return false;
 	}
 
 	QTextDocument::FindFlags flags = 0;
 	Qt::CaseSensitivity cs = Qt::CaseInsensitive;
-	if(backwards){
+	if(backwards) {
 		flags |= QTextDocument::FindBackward;
 	}
-	if(matchCase){
+	if(matchCase) {
 		flags |= QTextDocument::FindCaseSensitively;
 		cs = Qt::CaseSensitive;
 	}
@@ -104,30 +101,30 @@ bool OutputTextEdit::findReplace(bool backwards, bool replace, bool matchCase, c
 	QTextCursor regionCursor = regionBounds();
 
 	QTextCursor selCursor = textCursor();
-	if(selCursor.selectedText().compare(searchstr, cs) == 0){
-		if(replace){
+	if(selCursor.selectedText().compare(searchstr, cs) == 0) {
+		if(replace) {
 			selCursor.insertText(replacestr);
 			selCursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor, replacestr.length());
 			setTextCursor(selCursor);
 			ensureCursorVisible();
 			return true;
 		}
-		if(backwards){
+		if(backwards) {
 			selCursor.setPosition(qMin(selCursor.position(), selCursor.anchor()));
-		}else{
+		} else {
 			selCursor.setPosition(qMax(selCursor.position(), selCursor.anchor()));
 		}
 	}
 
 	QTextCursor findcursor = document()->find(searchstr, selCursor, flags);
-	if(findcursor.isNull() || !(findcursor.anchor() >= regionCursor.anchor() && findcursor.position() <= regionCursor.position())){
-		if(backwards){
+	if(findcursor.isNull() || !(findcursor.anchor() >= regionCursor.anchor() && findcursor.position() <= regionCursor.position())) {
+		if(backwards) {
 			selCursor.setPosition(regionCursor.position());
-		}else{
+		} else {
 			selCursor.setPosition(regionCursor.anchor());
 		}
 		findcursor = document()->find(searchstr, selCursor, flags);
-		if(findcursor.isNull() || !(findcursor.anchor() >= regionCursor.anchor() && findcursor.position() <= regionCursor.position())){
+		if(findcursor.isNull() || !(findcursor.anchor() >= regionCursor.anchor() && findcursor.position() <= regionCursor.position())) {
 			return false;
 		}
 	}
@@ -136,31 +133,29 @@ bool OutputTextEdit::findReplace(bool backwards, bool replace, bool matchCase, c
 	return true;
 }
 
-bool OutputTextEdit::replaceAll(const QString &searchstr, const QString &replacestr, bool matchCase)
-{
+bool OutputTextEdit::replaceAll(const QString &searchstr, const QString &replacestr, bool matchCase) {
 	QTextCursor cursor =  regionBounds();
 	int end = cursor.position();
 	QString cursel = cursor.selectedText();
 	if(cursor.anchor() == cursor.position() ||
-	   cursel == searchstr ||
-	   cursel == replacestr)
-	{
+	        cursel == searchstr ||
+	        cursel == replacestr) {
 		cursor.movePosition(QTextCursor::Start);
 		QTextCursor tmp(cursor);
 		tmp.movePosition(QTextCursor::End);
 		end = tmp.position();
-	}else{
+	} else {
 		cursor.setPosition(qMin(cursor.anchor(), cursor.position()));
 	}
 	QTextDocument::FindFlags flags = 0;
-	if(matchCase){
+	if(matchCase) {
 		flags = QTextDocument::FindCaseSensitively;
 	}
 	int diff = replacestr.length() - searchstr.length();
 	int count = 0;
-	while(true){
+	while(true) {
 		cursor = document()->find(searchstr, cursor, flags);
-		if(cursor.isNull() || cursor.position() > end){
+		if(cursor.isNull() || cursor.position() > end) {
 			break;
 		}
 		cursor.insertText(replacestr);
@@ -168,29 +163,27 @@ bool OutputTextEdit::replaceAll(const QString &searchstr, const QString &replace
 		++count;
 		QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 	}
-	if(count == 0){
+	if(count == 0) {
 		return false;
 	}
 	return true;
 }
 
-void OutputTextEdit::setDrawWhitespace(bool drawWhitespace)
-{
+void OutputTextEdit::setDrawWhitespace(bool drawWhitespace) {
 	m_drawWhitespace = drawWhitespace;
 	QTextOption textOption = document()->defaultTextOption();
-	if(drawWhitespace){
+	if(drawWhitespace) {
 		textOption.setFlags(QTextOption::ShowTabsAndSpaces | QTextOption::AddSpaceForLineAndParagraphSeparators);
-	}else{
+	} else {
 		textOption.setFlags(0);
 	}
 	document()->setDefaultTextOption(textOption);
 }
 
-void OutputTextEdit::paintEvent(QPaintEvent *e)
-{
+void OutputTextEdit::paintEvent(QPaintEvent *e) {
 	QPointF offset = contentOffset();
 
-	if(!m_entireRegion){
+	if(!m_entireRegion) {
 		QPainter painter(viewport());
 		painter.setBrush(palette().color(QPalette::Highlight).lighter(160));
 		painter.setPen(Qt::NoPen);
@@ -211,13 +204,13 @@ void OutputTextEdit::paintEvent(QPaintEvent *e)
 		// Draw selection
 		qreal top;
 		QRectF rect;
-		if(startBlock.blockNumber() == endBlock.blockNumber() && startLine.lineNumber() == endLine.lineNumber()){
+		if(startBlock.blockNumber() == endBlock.blockNumber() && startLine.lineNumber() == endLine.lineNumber()) {
 			top = blockBoundingGeometry(startBlock).translated(offset).top();
 			rect = startLine.naturalTextRect().translated(offset.x() - 0.5, top);
 			rect.setLeft(startLine.cursorToX(startLinePos) - 0.5);
 			rect.setRight(endLine.cursorToX(endLinePos));
 			painter.drawRect(rect);
-		}else{
+		} else {
 			// Draw selection on start line
 			top = blockBoundingGeometry(startBlock).translated(offset).top();
 			rect = startLine.naturalTextRect().translated(offset.x() - 0.5, top);
@@ -227,12 +220,12 @@ void OutputTextEdit::paintEvent(QPaintEvent *e)
 			// Draw selections inbetween
 			QTextBlock block = startBlock;
 			int lineNo = startLine.lineNumber() + 1;
-			while(!(block.blockNumber() == endBlock.blockNumber() && lineNo == endLine.lineNumber())){
-				if(block.isValid() && lineNo < block.lineCount()){
+			while(!(block.blockNumber() == endBlock.blockNumber() && lineNo == endLine.lineNumber())) {
+				if(block.isValid() && lineNo < block.lineCount()) {
 					painter.drawRect(block.layout()->lineAt(lineNo).naturalTextRect().translated(offset.x() - 0.5, top));
 				}
 				++lineNo;
-				if(lineNo >= block.lineCount()){
+				if(lineNo >= block.lineCount()) {
 					block = block.next();
 					top = blockBoundingGeometry(block).translated(offset).top();
 					lineNo = 0;
@@ -249,7 +242,7 @@ void OutputTextEdit::paintEvent(QPaintEvent *e)
 
 	QPlainTextEdit::paintEvent(e);
 
-	if(m_drawWhitespace){
+	if(m_drawWhitespace) {
 		QTextBlock block = firstVisibleBlock();
 		qreal top = blockBoundingGeometry(block).translated(offset).top();
 		qreal bottom = top + blockBoundingRect(block).height();
@@ -260,15 +253,15 @@ void OutputTextEdit::paintEvent(QPaintEvent *e)
 		QChar paragraph((ushort)0x00b6);
 
 		// block.next().isValid(): don't draw line break on last block
-		while(block.isValid() && block.next().isValid() && top <= e->rect().bottom()){
-			if(block.isVisible() && bottom >= e->rect().top()){
+		while(block.isValid() && block.next().isValid() && top <= e->rect().bottom()) {
+			if(block.isVisible() && bottom >= e->rect().top()) {
 				QTextLayout *layout = block.layout();
 				// Draw hard line breaks (i.e. those not due to word wrapping)
 				QTextLine line = layout->lineAt(layout->lineCount() - 1);
 				QRectF lineRect = line.naturalTextRect().translated(offset.x(), top);
-				if(line.textLength() == 0){
+				if(line.textLength() == 0) {
 					painter.drawText(QPointF(lineRect.right(), lineRect.top() + line.ascent()), paragraph);
-				}else{
+				} else {
 					painter.drawText(QPointF(lineRect.right(), lineRect.top() + line.ascent()), visualArrow);
 				}
 			}
@@ -279,26 +272,25 @@ void OutputTextEdit::paintEvent(QPaintEvent *e)
 	}
 }
 
-void OutputTextEdit::saveRegionBounds()
-{
+void OutputTextEdit::saveRegionBounds() {
 	QTextCursor c = textCursor();
-	if(hasFocus()){
+	if(hasFocus()) {
 		bool dorepaint = m_regionCursor.hasSelection() &&
-				!((m_regionCursor.anchor() == c.anchor() && m_regionCursor.position() == c.position()) ||
-				  (m_regionCursor.anchor() == c.position() && m_regionCursor.position() == c.anchor()));
+		                 !((m_regionCursor.anchor() == c.anchor() && m_regionCursor.position() == c.position()) ||
+		                   (m_regionCursor.anchor() == c.position() && m_regionCursor.position() == c.anchor()));
 		m_regionCursor = c;
-		if(dorepaint){
+		if(dorepaint) {
 			viewport()->repaint();
 		}
 		// If only one word is selected, don't treat it as a region
-		if(!m_regionCursor.selectedText().contains(QRegExp("\\s"))){
+		if(!m_regionCursor.selectedText().contains(QRegExp("\\s"))) {
 			m_regionCursor.clearSelection();
 		}
 	}
 	c.movePosition(QTextCursor::Start);
 	c.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
 	// If nothing is selected, set the region to the entire contents
-	if(!m_regionCursor.hasSelection()){
+	if(!m_regionCursor.hasSelection()) {
 		m_regionCursor.setPosition(c.anchor());
 		m_regionCursor.setPosition(c.position(), QTextCursor::KeepAnchor);
 	}
