@@ -36,13 +36,16 @@
 Q_DECLARE_METATYPE(Source*)
 
 // Only to silence "QVariant::save: unable to save type 'Source*'" warning
-QDataStream& operator<<(QDataStream& ds, const Source*&) { return ds; }
-QDataStream& operator>>(QDataStream& ds, Source*&) {return ds; }
+QDataStream& operator<<(QDataStream& ds, const Source*&) {
+	return ds;
+}
+QDataStream& operator>>(QDataStream& ds, Source*&) {
+	return ds;
+}
 
 
 SourceManager::SourceManager(const UI_MainWindow& _ui)
-	: ui(_ui)
-{
+	: ui(_ui) {
 	m_recentMenu = new QMenu(MAIN);
 	ui.actionSourceRecent->setMenu(m_recentMenu);
 
@@ -66,30 +69,28 @@ SourceManager::SourceManager(const UI_MainWindow& _ui)
 	qRegisterMetaTypeStreamOperators<Source*>("Source*");
 }
 
-SourceManager::~SourceManager()
-{
+SourceManager::~SourceManager() {
 	clearSources();
 }
 
-void SourceManager::addSources(const QStringList& files)
-{
+void SourceManager::addSources(const QStringList& files) {
 	QString failed;
 	QListWidgetItem* item = nullptr;
 	QStringList recentItems = MAIN->getConfig()->getSetting<VarSetting<QStringList>>("recentitems")->getValue();
-	for(const QString& filename : files){
-		if(!QFile(filename).exists()){
+	for(const QString& filename : files) {
+		if(!QFile(filename).exists()) {
 			failed += "\n\t" + filename;
 			continue;
 		}
 		bool contains = false;
-		for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row){
-			if(ui.listWidgetSources->item(row)->toolTip() == filename){
+		for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row) {
+			if(ui.listWidgetSources->item(row)->toolTip() == filename) {
 				item = ui.listWidgetSources->item(row);
 				contains = true;
 				break;
 			}
 		}
-		if(contains){
+		if(contains) {
 			continue;
 		}
 		item = new QListWidgetItem(QFileInfo(filename).fileName(), ui.listWidgetSources);
@@ -101,33 +102,30 @@ void SourceManager::addSources(const QStringList& files)
 		recentItems.prepend(filename);
 	}
 	MAIN->getConfig()->getSetting<VarSetting<QStringList>>("recentitems")->setValue(recentItems);
-	if(item){
+	if(item) {
 		ui.listWidgetSources->blockSignals(true);
 		ui.listWidgetSources->clearSelection();
 		ui.listWidgetSources->blockSignals(false);
 		ui.listWidgetSources->setCurrentItem(item);
 	}
-	if(!failed.isEmpty()){
+	if(!failed.isEmpty()) {
 		QMessageBox::critical(MAIN, _("Unable to open files"), _("The following files could not be opened:%1").arg(failed));
 	}
 }
 
-QList<Source*> SourceManager::getSelectedSources() const
-{
+QList<Source*> SourceManager::getSelectedSources() const {
 	QList<Source*> selectedSources;
-	for(const QListWidgetItem* item : ui.listWidgetSources->selectedItems())
-	{
+	for(const QListWidgetItem* item : ui.listWidgetSources->selectedItems()) {
 		selectedSources.append(item->data(Qt::UserRole).value<Source*>());
 	}
 	return selectedSources;
 }
 
-void SourceManager::prepareSourcesMenu()
-{
+void SourceManager::prepareSourcesMenu() {
 	// Build recent menu
 	m_recentMenu->clear();
-	for(const QString& filename : MAIN->getConfig()->getSetting<VarSetting<QStringList>>("recentitems")->getValue()){
-		if(QFile(filename).exists()){
+	for(const QString& filename : MAIN->getConfig()->getSetting<VarSetting<QStringList>>("recentitems")->getValue()) {
+		if(QFile(filename).exists()) {
 			QAction* action = new QAction(QFileInfo(filename).fileName(), m_recentMenu);
 			action->setToolTip(filename);
 			connect(action, SIGNAL(triggered()), this, SLOT(openRecentItem()));
@@ -140,17 +138,16 @@ void SourceManager::prepareSourcesMenu()
 	ui.actionSourcePaste->setEnabled(!QApplication::clipboard()->pixmap().isNull());
 }
 
-void SourceManager::openSources()
-{
+void SourceManager::openSources() {
 	QList<Source*> current = getSelectedSources();
 	QString dir;
-	if(!current.isEmpty() && !current.front()->isTemp){
+	if(!current.isEmpty() && !current.front()->isTemp) {
 		dir = QFileInfo(current.front()->path).absolutePath();
-	}else{
+	} else {
 		dir = Utils::documentsFolder();
 	}
 	QSet<QString> formats;
-	for(const QByteArray& format : QImageReader::supportedImageFormats()){
+	for(const QByteArray& format : QImageReader::supportedImageFormats()) {
 		formats.insert(QString("*.%1").arg(QString(format).toLower()));
 	}
 	formats.insert("*.pdf");
@@ -158,16 +155,14 @@ void SourceManager::openSources()
 	addSources(QFileDialog::getOpenFileNames(MAIN, _("Select Files"), dir, filter));
 }
 
-void SourceManager::openRecentItem()
-{
+void SourceManager::openRecentItem() {
 	const QString& filename = qobject_cast<QAction*>(QObject::sender())->toolTip();
 	addSources(QStringList() << filename);
 }
 
-void SourceManager::pasteClipboard()
-{
+void SourceManager::pasteClipboard() {
 	QPixmap pixmap = QApplication::clipboard()->pixmap();
-	if(pixmap.isNull()){
+	if(pixmap.isNull()) {
 		QMessageBox::critical(MAIN, _("Clipboard Error"),  _("Failed to read the clipboard."));
 		return;
 	}
@@ -176,17 +171,15 @@ void SourceManager::pasteClipboard()
 	savePixmap(pixmap, displayname);
 }
 
-void SourceManager::addSourceImage(const QImage& image)
-{
+void SourceManager::addSourceImage(const QImage& image) {
 	++m_pasteCount;
 	QString displayname = _("Pasted %1").arg(m_pasteCount);
 	savePixmap(QPixmap::fromImage(image), displayname);
 }
 
-void SourceManager::takeScreenshot()
-{
+void SourceManager::takeScreenshot() {
 	QPixmap pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
-	if(pixmap.isNull()){
+	if(pixmap.isNull()) {
 		QMessageBox::critical(MAIN, _("Screenshot Error"),  _("Failed to take screenshot."));
 		return;
 	}
@@ -195,23 +188,22 @@ void SourceManager::takeScreenshot()
 	savePixmap(pixmap, displayname);
 }
 
-void SourceManager::savePixmap(const QPixmap& pixmap, const QString& displayname)
-{
+void SourceManager::savePixmap(const QPixmap& pixmap, const QString& displayname) {
 	MAIN->pushState(MainWindow::State::Busy, _("Saving image..."));
 	QString filename;
 	bool success = true;
 	QTemporaryFile tmpfile(QDir::temp().absoluteFilePath("gimagereader_XXXXXX.png"));
-	if(!tmpfile.open()){
+	if(!tmpfile.open()) {
 		success = false;
-	}else{
+	} else {
 		tmpfile.setAutoRemove(false);
 		filename = tmpfile.fileName();
 		success = pixmap.save(filename);
 	}
 	MAIN->popState();
-	if(!success){
+	if(!success) {
 		QMessageBox::critical(MAIN, _("Cannot Write File"),  _("Could not write to %1.").arg(filename));
-	}else{
+	} else {
 		QListWidgetItem* item = new QListWidgetItem(displayname, ui.listWidgetSources);
 		item->setToolTip(filename);
 		Source* source = new Source(filename, displayname, true);
@@ -224,8 +216,7 @@ void SourceManager::savePixmap(const QPixmap& pixmap, const QString& displayname
 	}
 }
 
-void SourceManager::removeSource(bool deleteFile)
-{
+void SourceManager::removeSource(bool deleteFile) {
 	QString paths;
 	for(const QListWidgetItem* item : ui.listWidgetSources->selectedItems()) {
 		paths += QString("\n") + item->data(Qt::UserRole).value<Source*>()->path;
@@ -233,16 +224,15 @@ void SourceManager::removeSource(bool deleteFile)
 	if(paths.isEmpty()) {
 		return;
 	}
-	if(deleteFile && QMessageBox::Yes != QMessageBox::question(MAIN, _("Delete File?"), _("The following files will be deleted:%1").arg(paths), QMessageBox::Yes, QMessageBox::No)){
+	if(deleteFile && QMessageBox::Yes != QMessageBox::question(MAIN, _("Delete File?"), _("The following files will be deleted:%1").arg(paths), QMessageBox::Yes, QMessageBox::No)) {
 		return;
 	}
 	// Avoid multiple sourceChanged emissions when removing items
 	ui.listWidgetSources->blockSignals(true);
-	for(const QListWidgetItem* item : ui.listWidgetSources->selectedItems())
-	{
+	for(const QListWidgetItem* item : ui.listWidgetSources->selectedItems()) {
 		Source* source = item->data(Qt::UserRole).value<Source*>();
 		m_fsWatcher.removePath(source->path);
-		if(deleteFile || source->isTemp){
+		if(deleteFile || source->isTemp) {
 			QFile(source->path).remove();
 		}
 		delete source;
@@ -256,14 +246,13 @@ void SourceManager::removeSource(bool deleteFile)
 }
 
 
-void SourceManager::clearSources()
-{
-	if(!m_fsWatcher.files().isEmpty()){
+void SourceManager::clearSources() {
+	if(!m_fsWatcher.files().isEmpty()) {
 		m_fsWatcher.removePaths(m_fsWatcher.files());
 	}
-	for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row){
+	for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row) {
 		Source* source = ui.listWidgetSources->item(row)->data(Qt::UserRole).value<Source*>();
-		if(source->isTemp){
+		if(source->isTemp) {
 			QFile(source->path).remove();
 		}
 		delete source;
@@ -272,8 +261,7 @@ void SourceManager::clearSources()
 	ui.listWidgetSources->clear();
 }
 
-void SourceManager::currentSourceChanged()
-{
+void SourceManager::currentSourceChanged() {
 	bool enabled = !ui.listWidgetSources->selectedItems().isEmpty();
 	ui.actionSourceRemove->setEnabled(enabled);
 	ui.actionSourceDelete->setEnabled(enabled);
@@ -281,13 +269,12 @@ void SourceManager::currentSourceChanged()
 	emit sourceChanged();
 }
 
-void SourceManager::fileChanged(const QString& filename)
-{
-	if(!QFile(filename).exists()){
-		for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row){
+void SourceManager::fileChanged(const QString& filename) {
+	if(!QFile(filename).exists()) {
+		for(int row = 0, nRows = ui.listWidgetSources->count(); row < nRows; ++row) {
 			QListWidgetItem* item = ui.listWidgetSources->item(row);
 			Source* source = item->data(Qt::UserRole).value<Source*>();
-			if(source->path == filename){
+			if(source->path == filename) {
 				QMessageBox::warning(MAIN, _("Missing File"), _("The following file has been deleted or moved:\n%1").arg(filename));
 				delete item;
 				delete source;
@@ -298,15 +285,14 @@ void SourceManager::fileChanged(const QString& filename)
 	}
 }
 
-bool SourceManager::eventFilter(QObject *object, QEvent *event)
-{
-	if(event->type() == QEvent::DragEnter){
+bool SourceManager::eventFilter(QObject *object, QEvent *event) {
+	if(event->type() == QEvent::DragEnter) {
 		QDragEnterEvent* dragEnterEvent = static_cast<QDragEnterEvent*>(event);
-		if(Utils::handleSourceDragEvent(dragEnterEvent->mimeData())){
+		if(Utils::handleSourceDragEvent(dragEnterEvent->mimeData())) {
 			dragEnterEvent->acceptProposedAction();
 		}
 		return true;
-	}else if(event->type() == QEvent::Drop){
+	} else if(event->type() == QEvent::Drop) {
 		QDropEvent* dropEvent = static_cast<QDropEvent*>(event);
 		Utils::handleSourceDropEvent(dropEvent->mimeData());
 		return true;
