@@ -42,8 +42,7 @@
 #include "Utils.hh"
 
 TessdataManager::TessdataManager(QWidget *parent)
-	: QDialog(parent)
-{
+	: QDialog(parent) {
 	setWindowTitle(_("Tessdata Manager"));
 	setLayout(new QVBoxLayout());
 	layout()->addWidget(new QLabel(_("Manage installed languages:")));
@@ -54,37 +53,38 @@ TessdataManager::TessdataManager(QWidget *parent)
 	QDialogButtonBox* bbox = new QDialogButtonBox();
 	bbox->addButton(QDialogButtonBox::Close);
 	bbox->addButton(QDialogButtonBox::Apply);
+	QPushButton* refreshButton = bbox->addButton(_("Refresh"), QDialogButtonBox::ActionRole);
 	connect(bbox->button(QDialogButtonBox::Apply), SIGNAL(clicked(bool)), this, SLOT(applyChanges()));
 	connect(bbox, SIGNAL(accepted()), this, SLOT(accept()));
 	connect(bbox, SIGNAL(rejected()), this, SLOT(reject()));
+	connect(refreshButton, SIGNAL(clicked(bool)), this, SLOT(refresh()));
 	layout()->addWidget(bbox);
 	setFixedWidth(320);
 }
 
-bool TessdataManager::setup()
-{
+bool TessdataManager::setup() {
 #ifdef Q_OS_LINUX
-	QDBusConnectionInterface* iface = QDBusConnection::sessionBus().interface();
-	iface->startService("org.freedesktop.PackageKit");
-	if(!iface->isServiceRegistered("org.freedesktop.PackageKit").value()){
-		QMessageBox::critical(MAIN, _("Error"), _("PackageKit is required for managing tesseract language packs, but it was not found. Please use the system package management software to manage the tesseract language packs."));
-		return false;
+	if(MAIN->getConfig()->useSystemDataLocations()) {
+		QDBusConnectionInterface* iface = QDBusConnection::sessionBus().interface();
+		iface->startService("org.freedesktop.PackageKit");
+		if(!iface->isServiceRegistered("org.freedesktop.PackageKit").value()) {
+			QMessageBox::critical(MAIN, _("Error"), _("PackageKit is required for managing system-wide tesseract language packs, but it was not found. Please use the system package management software to manage the tesseract language packs, or switch to use the user tessdata path in the configuration dialog."));
+			return false;
+		}
 	}
 #endif
 	MAIN->pushState(MainWindow::State::Busy, _("Fetching available languages"));
 	QString messages;
 	bool success = fetchLanguageList(messages);
 	MAIN->popState();
-	if(!success)
-	{
+	if(!success) {
 		QMessageBox::critical(MAIN, _("Error"), _("Failed to fetch list of available languages: %1").arg(messages));
 		return false;
 	}
 	return true;
 }
 
-bool TessdataManager::fetchLanguageList(QString& messages)
-{
+bool TessdataManager::fetchLanguageList(QString& messages) {
 	QByteArray data = Utils::download(QUrl("https://api.github.com/repos/tesseract-ocr/tessdata/contents"), messages);
 
 	if(data.isEmpty()) {
@@ -152,86 +152,101 @@ bool TessdataManager::fetchLanguageList(QString& messages)
 	return true;
 }
 
-void TessdataManager::applyChanges()
-{
+void TessdataManager::applyChanges() {
 	MAIN->pushState(MainWindow::State::Busy, _("Applying changes..."));
 	setEnabled(false);
 	setCursor(Qt::WaitCursor);
 	QString errorMsg;
 	QStringList availableLanguages = MAIN->getRecognizer()->getAvailableLanguages();
-	QDir tessDataDir(MAIN->getRecognizer()->getTessdataDir());
-#ifdef Q_OS_LINUX
-	QStringList installFiles;
-	QStringList removeFiles;
-	for(int row = 0, nRows = m_languageList->count(); row < nRows; ++row) {
-		QListWidgetItem* item = m_languageList->item(row);
-		QString prefix = item->data(Qt::UserRole).toString();
-		if(item->checkState() == Qt::Checked && !availableLanguages.contains(prefix)) {
-			installFiles.append(tessDataDir.absoluteFilePath(QString("%1.traineddata").arg(prefix)));
-		} else if(item->checkState() != Qt::Checked && availableLanguages.contains(prefix)) {
-			removeFiles.append(tessDataDir.absoluteFilePath(QString("%1.traineddata").arg(prefix)));
-		}
-	}
-
-	if(!installFiles.isEmpty()) {
-		QDBusMessage req = QDBusMessage::createMethodCall("org.freedesktop.PackageKit", "/org/freedesktop/PackageKit", "org.freedesktop.PackageKit.Modify", "InstallProvideFiles");
-		req.setArguments(QList<QVariant>() << QVariant::fromValue((quint32)winId()) << QVariant::fromValue(installFiles) << QVariant::fromValue(QString("always")));
-		QDBusMessage reply = QDBusConnection::sessionBus().call(req, QDBus::BlockWithGui, 3600000);
-		if(reply.type() == QDBusMessage::ErrorMessage) {
-			errorMsg = reply.errorMessage();
-		}
-	}
-	if(errorMsg.isEmpty() && !removeFiles.isEmpty()) {
-		QDBusMessage req = QDBusMessage::createMethodCall("org.freedesktop.PackageKit", "/org/freedesktop/PackageKit", "org.freedesktop.PackageKit.Modify", "RemovePackageByFiles");
-		req.setArguments(QList<QVariant>() << QVariant::fromValue((quint32)winId()) << QVariant::fromValue(removeFiles) << QVariant::fromValue(QString("always")));
-		QDBusMessage reply = QDBusConnection::sessionBus().call(req, QDBus::BlockWithGui, 3600000);
-		if(reply.type() == QDBusMessage::ErrorMessage) {
-			errorMsg = reply.errorMessage();
-		}
-	}
+	QDir tessDataDir(MAIN->getConfig()->tessdataLocation());
+#ifdef Q_OS_WIN
+	bool isWindows = true;
 #else
-	QStringList errors;
-	for(int row = 0, nRows = m_languageList->count(); row < nRows; ++row) {
-		QListWidgetItem* item = m_languageList->item(row);
-		QString prefix = item->data(Qt::UserRole).toString();
-		if(item->checkState() == Qt::Checked && !availableLanguages.contains(prefix)) {
-			for(const LangFile& langFile : m_languageFiles.value(prefix)) {
-				if(!QFile(tessDataDir.absoluteFilePath(langFile.name)).exists()) {
-					MAIN->pushState(MainWindow::State::Busy, _("Downloading %1...").arg(langFile.name));
-					QString messages;
-					QByteArray data = Utils::download(QUrl(langFile.url), messages);
-					QFile file(tessDataDir.absoluteFilePath(langFile.name));
-					if(data.isEmpty() || !file.open(QIODevice::WriteOnly)) {
-						errors.append(langFile.name);
-					} else {
-						file.write(data);
-					}
-					MAIN->popState();
-				}
+	bool isWindows = false;
+#endif
+	if(!isWindows && MAIN->getConfig()->useSystemDataLocations()) {
+		// Place this in a ifdef since DBus stuff cannot be compiled on Windows
+#ifdef Q_OS_LINUX
+		QStringList installFiles;
+		QStringList removeFiles;
+		for(int row = 0, nRows = m_languageList->count(); row < nRows; ++row) {
+			QListWidgetItem* item = m_languageList->item(row);
+			QString prefix = item->data(Qt::UserRole).toString();
+			if(item->checkState() == Qt::Checked && !availableLanguages.contains(prefix)) {
+				installFiles.append(tessDataDir.absoluteFilePath(QString("%1.traineddata").arg(prefix)));
+			} else if(item->checkState() != Qt::Checked && availableLanguages.contains(prefix)) {
+				removeFiles.append(tessDataDir.absoluteFilePath(QString("%1.traineddata").arg(prefix)));
 			}
-		} else if(item->checkState() != Qt::Checked && availableLanguages.contains(prefix)) {
-			foreach(const QString& file, tessDataDir.entryList(QStringList() << prefix + ".*")) {
-				if(!QFile(tessDataDir.absoluteFilePath(file)).remove()) {
-					errors.append(file);
+		}
+
+		if(!installFiles.isEmpty()) {
+			QDBusMessage req = QDBusMessage::createMethodCall("org.freedesktop.PackageKit", "/org/freedesktop/PackageKit", "org.freedesktop.PackageKit.Modify", "InstallProvideFiles");
+			req.setArguments(QList<QVariant>() << QVariant::fromValue((quint32)winId()) << QVariant::fromValue(installFiles) << QVariant::fromValue(QString("always")));
+			QDBusMessage reply = QDBusConnection::sessionBus().call(req, QDBus::BlockWithGui, 3600000);
+			if(reply.type() == QDBusMessage::ErrorMessage) {
+				errorMsg = reply.errorMessage();
+			}
+		}
+		if(errorMsg.isEmpty() && !removeFiles.isEmpty()) {
+			QDBusMessage req = QDBusMessage::createMethodCall("org.freedesktop.PackageKit", "/org/freedesktop/PackageKit", "org.freedesktop.PackageKit.Modify", "RemovePackageByFiles");
+			req.setArguments(QList<QVariant>() << QVariant::fromValue((quint32)winId()) << QVariant::fromValue(removeFiles) << QVariant::fromValue(QString("always")));
+			QDBusMessage reply = QDBusConnection::sessionBus().call(req, QDBus::BlockWithGui, 3600000);
+			if(reply.type() == QDBusMessage::ErrorMessage) {
+				errorMsg = reply.errorMessage();
+			}
+		}
+#endif
+	} else {
+		QStringList errors;
+		if(!QDir().mkpath(tessDataDir.absolutePath())) {
+			errors.append(_("Failed to create directory for tessdata files."));
+		} else {
+			for(int row = 0, nRows = m_languageList->count(); row < nRows; ++row) {
+				QListWidgetItem* item = m_languageList->item(row);
+				QString prefix = item->data(Qt::UserRole).toString();
+				if(item->checkState() == Qt::Checked && !availableLanguages.contains(prefix)) {
+					for(const LangFile& langFile : m_languageFiles.value(prefix)) {
+						if(!QFile(tessDataDir.absoluteFilePath(langFile.name)).exists()) {
+							MAIN->pushState(MainWindow::State::Busy, _("Downloading %1...").arg(langFile.name));
+							QString messages;
+							QByteArray data = Utils::download(QUrl(langFile.url), messages);
+							QFile file(tessDataDir.absoluteFilePath(langFile.name));
+							if(data.isEmpty() || !file.open(QIODevice::WriteOnly)) {
+								errors.append(langFile.name);
+							} else {
+								file.write(data);
+							}
+							MAIN->popState();
+						}
+					}
+				} else if(item->checkState() != Qt::Checked && availableLanguages.contains(prefix)) {
+					foreach(const QString& file, tessDataDir.entryList(QStringList() << prefix + ".*")) {
+						if(!QFile(tessDataDir.absoluteFilePath(file)).remove()) {
+							errors.append(file);
+						}
+					}
 				}
 			}
 		}
+		if(!errors.isEmpty()) {
+			errorMsg = _("The following files could not be downloaded or removed:\n%1\n\nCheck the connectivity and directory permissions.").arg(errors.join("\n"));
+		}
 	}
-	if(!errors.isEmpty()) {
-		errorMsg = _("The following files could not be downloaded or removed:\n%1\n\nCheck the connectivity and directory permissions.").arg(errors.join("\n"));
-	}
-#endif
 	unsetCursor();
 	setEnabled(true);
 	MAIN->popState();
+	refresh();
+	if(!errorMsg.isEmpty()) {
+		QMessageBox::critical(this, _("Error"), errorMsg);
+	}
+}
+
+void TessdataManager::refresh() {
 	MAIN->getRecognizer()->updateLanguagesMenu();
-	availableLanguages = MAIN->getRecognizer()->getAvailableLanguages();
+	QStringList availableLanguages = MAIN->getRecognizer()->getAvailableLanguages();
 	for(int row = 0, nRows = m_languageList->count(); row < nRows; ++row) {
 		QListWidgetItem* item = m_languageList->item(row);
 		QString prefix = item->data(Qt::UserRole).toString();
 		item->setCheckState(availableLanguages.contains(prefix) ? Qt::Checked : Qt::Unchecked);
 	}
-	if(!errorMsg.isEmpty()) {
-		QMessageBox::critical(this, _("Error"), errorMsg);
-	}
 }
