@@ -29,30 +29,26 @@
 #include <QStringList>
 #include <sane/saneopts.h>
 
-void ScannerSane::init()
-{
+void ScannerSane::init() {
 	moveToThread(&m_thread);
 	connect(&m_thread, SIGNAL(started()), this, SLOT(run()));
 	m_thread.start();
 }
 
-void ScannerSane::redetect(){
+void ScannerSane::redetect() {
 	m_requestQueue.enqueue({Request::Type::Redetect, 0});
 }
 
-void ScannerSane::scan(const Params& params)
-{
+void ScannerSane::scan(const Params& params) {
 	Q_ASSERT(params.depth == 8); // only 8 supported
 	m_requestQueue.enqueue({Request::Type::StartScan, new ScanJob(params)});
 }
 
-void ScannerSane::cancel()
-{
+void ScannerSane::cancel() {
 	m_requestQueue.enqueue({Request::Type::Cancel, 0});
 }
 
-void ScannerSane::close()
-{
+void ScannerSane::close() {
 	m_requestQueue.enqueue({Request::Type::Quit, 0});
 	m_thread.quit();
 	m_thread.wait();
@@ -60,13 +56,12 @@ void ScannerSane::close()
 
 /***************************** Thread functions ******************************/
 
-void ScannerSane::run()
-{
+void ScannerSane::run() {
 	// Initialize sane
 	SANE_Int version_code;
 	SANE_Status status = sane_init(&version_code, nullptr);
 	qDebug("sane_init() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		qWarning("Unable to initialize SANE backend: %s", sane_strstatus(status));
 		emit initFailed();
 		return;
@@ -77,37 +72,37 @@ void ScannerSane::run()
 	doRedetect();
 
 	bool need_redetect = false;
-	while(true){
+	while(true) {
 		// Fetch request if requests are pending or wait for request if idle
-		if(!m_requestQueue.empty() || m_state == State::IDLE){
+		if(!m_requestQueue.empty() || m_state == State::IDLE) {
 			Request request = m_requestQueue.dequeue();
-			if(request.type == Request::Type::Redetect){
+			if(request.type == Request::Type::Redetect) {
 				need_redetect = true;
-			}else if(request.type == Request::Type::StartScan){
+			} else if(request.type == Request::Type::StartScan) {
 				setState(State::OPEN);
 				m_job = request.job;
-			}else if(request.type == Request::Type::Cancel){
+			} else if(request.type == Request::Type::Cancel) {
 				failScan(_("Scan canceled"));
-			}else if(request.type == Request::Type::Quit){
+			} else if(request.type == Request::Type::Quit) {
 				doStop();
 				break;
 			}
 		}
 		// Perform operations according to state
-		if(m_state == State::IDLE){
-			if(need_redetect){
+		if(m_state == State::IDLE) {
+			if(need_redetect) {
 				doRedetect();
 				need_redetect = false;
 			}
-		}else if(m_state == State::OPEN){
+		} else if(m_state == State::OPEN) {
 			doOpen();
-		}else if(m_state == State::SET_OPTIONS){
+		} else if(m_state == State::SET_OPTIONS) {
 			doSetOptions();
-		}else if(m_state == State::START){
+		} else if(m_state == State::START) {
 			doStart();
-		}else if(m_state == State::GET_PARAMETERS){
+		} else if(m_state == State::GET_PARAMETERS) {
 			doGetParameters();
-		}else if(m_state == State::READ){
+		} else if(m_state == State::READ) {
 			doRead();
 		}
 	}
@@ -117,31 +112,28 @@ void ScannerSane::run()
 //	sane_exit();
 }
 
-void ScannerSane::setState(State state)
-{
+void ScannerSane::setState(State state) {
 	m_state = state;
 	emit scanStateChanged(m_state);
 }
 
-void ScannerSane::failScan(const QString& errorString)
-{
+void ScannerSane::failScan(const QString& errorString) {
 	doStop();
 	emit scanFailed(errorString);
 }
 
-void ScannerSane::doRedetect()
-{
+void ScannerSane::doRedetect() {
 	QList<Device> devices;
 
 	const SANE_Device** device_list = nullptr;
 	SANE_Status status = sane_get_devices(&device_list, false);
 	qDebug("sane_get_devices() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		qWarning("Unable to get SANE devices: %s", sane_strstatus(status));
-	}else{
-		for(int i = 0; device_list[i] != nullptr; ++i){
+	} else {
+		for(int i = 0; device_list[i] != nullptr; ++i) {
 			qDebug("Device: name=\"%s\" vendor=\"%s\" model=\"%s\" type=\"%s\"",
-				   device_list[i]->name, device_list[i]->vendor, device_list[i]->model, device_list[i]->type);
+			       device_list[i]->name, device_list[i]->vendor, device_list[i]->model, device_list[i]->type);
 
 			Device scan_device;
 			scan_device.name = device_list[i]->name;
@@ -161,16 +153,15 @@ void ScannerSane::doRedetect()
 	emit devicesDetected(devices);
 }
 
-void ScannerSane::doOpen()
-{
-	if(m_job->params.device.isEmpty()){
+void ScannerSane::doOpen() {
+	if(m_job->params.device.isEmpty()) {
 		failScan(_("No scanner specified"));
 		return;
 	}
 
 	SANE_Status status = sane_open(m_job->params.device.toLocal8Bit().data(), &m_job->handle);
 	qDebug("sane_open(\"%s\") -> %s", qPrintable(m_job->params.device), sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		m_job->handle = nullptr;
 		qWarning("Unable to get open device: %s", sane_strstatus(status));
 		failScan(_("Unable to connect to scanner"));
@@ -180,8 +171,7 @@ void ScannerSane::doOpen()
 	setState(State::SET_OPTIONS);
 }
 
-void ScannerSane::doSetOptions()
-{
+void ScannerSane::doSetOptions() {
 	qDebug("set_options");
 
 	int index = 0;
@@ -189,16 +179,16 @@ void ScannerSane::doSetOptions()
 	QMap<QString, int> options;
 
 	/* Build the option table */
-	while((option = sane_get_option_descriptor(m_job->handle, index)) != nullptr){
+	while((option = sane_get_option_descriptor(m_job->handle, index)) != nullptr) {
 		logOption(index, option);
 		if(
-			/* Ignore groups */
-			option->type != SANE_TYPE_GROUP &&
-			/* Option disabled */
-			(option->cap & SANE_CAP_INACTIVE) == 0 &&
-			/* Some options are unnamed (e.g. Option 0) */
-			option->name != nullptr && std::strlen(option->name) > 0
-		){
+		    /* Ignore groups */
+		    option->type != SANE_TYPE_GROUP &&
+		    /* Option disabled */
+		    (option->cap & SANE_CAP_INACTIVE) == 0 &&
+		    /* Some options are unnamed (e.g. Option 0) */
+		    option->name != nullptr && std::strlen(option->name) > 0
+		) {
 			options.insert(option->name, index);
 		}
 		++index;
@@ -208,7 +198,7 @@ void ScannerSane::doSetOptions()
 
 	/* Pick source */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_SOURCE, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		QStringList flatbed_sources = {
 			"Auto",
 			SANE_I18N("Auto"),
@@ -264,7 +254,7 @@ void ScannerSane::doSetOptions()
 
 	/* Scan mode (before resolution as it tends to affect that */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_MODE, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		/* The names of scan modes often used in drivers, as taken from the sane-backends source */
 		QStringList color_scan_modes = {
 			SANE_VALUE_SCAN_MODE_COLOR,
@@ -325,7 +315,7 @@ void ScannerSane::doSetOptions()
 
 	/* Disable compression, we will compress after scanning */
 	option = getOptionByName(options, m_job->handle, "compression", index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		QStringList disable_compression_names = {
 			SANE_I18N("None"),
 			SANE_I18N("none"),
@@ -339,10 +329,10 @@ void ScannerSane::doSetOptions()
 
 	/* Set resolution and bit depth */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_RESOLUTION, index);
-	if(option != nullptr){
+	if(option != nullptr) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.dpi, &m_job->params.dpi);
-		else{
+		else {
 			int dpi;
 			setIntOption(m_job->handle, option, index, m_job->params.dpi, &dpi);
 			m_job->params.dpi = dpi;
@@ -354,14 +344,14 @@ void ScannerSane::doSetOptions()
 
 	/* Always use maximum scan area - some scanners default to using partial areas.  This should be patched in sane-backends */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_BR_X, index);
-	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, SANE_UNFIX(option->constraint.range->max), nullptr);
 		else
 			setIntOption(m_job->handle, option, index, option->constraint.range->max, nullptr);
 	}
 	option = getOptionByName(options, m_job->handle, SANE_NAME_SCAN_BR_Y, index);
-	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option != nullptr && option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, SANE_UNFIX(option->constraint.range->max), nullptr);
 		else
@@ -370,14 +360,14 @@ void ScannerSane::doSetOptions()
 
 	/* Set page dimensions */
 	option = getOptionByName(options, m_job->handle, SANE_NAME_PAGE_WIDTH, index);
-	if(option != nullptr && m_job->params.page_width > 0.0){
+	if(option != nullptr && m_job->params.page_width > 0.0) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.page_width / 10.0, nullptr);
 		else
 			setIntOption(m_job->handle, option, index, m_job->params.page_width / 10, nullptr);
 	}
 	option = getOptionByName(options, m_job->handle, SANE_NAME_PAGE_HEIGHT, index);
-	if(option != nullptr && m_job->params.page_height > 0.0){
+	if(option != nullptr && m_job->params.page_height > 0.0) {
 		if(option->type == SANE_TYPE_FIXED)
 			setFixedOption(m_job->handle, option, index, m_job->params.page_height / 10.0, nullptr);
 		else
@@ -387,34 +377,32 @@ void ScannerSane::doSetOptions()
 	setState(State::START);
 }
 
-void ScannerSane::doStart()
-{
+void ScannerSane::doStart() {
 	SANE_Status status = sane_start(m_job->handle);
 	qDebug("sane_start -> %s", sane_strstatus(status));
-	if(status == SANE_STATUS_GOOD){
+	if(status == SANE_STATUS_GOOD) {
 		setState(State::GET_PARAMETERS);
-	}else{
+	} else {
 		doStop();
 		failScan(_("Unable to start scan"));
 	}
 }
 
-void ScannerSane::doGetParameters()
-{
+void ScannerSane::doGetParameters() {
 	SANE_Status status = sane_get_parameters(m_job->handle, &m_job->parameters);
 	qDebug("sane_get_parameters() -> %s", sane_strstatus(status));
-	if(status != SANE_STATUS_GOOD){
+	if(status != SANE_STATUS_GOOD) {
 		failScan(_("Error communicating with scanner"));
 		return;
 	}
 
 	qDebug("Parameters: format=%s last_frame=%s bytes_per_line=%d pixels_per_line=%d lines=%d depth=%d",
-			getFrameModeString(m_job->parameters.format).toLocal8Bit().data(),
-			m_job->parameters.last_frame ? "SANE_TRUE" : "SANE_FALSE",
-			m_job->parameters.bytes_per_line,
-			m_job->parameters.pixels_per_line,
-			m_job->parameters.lines,
-			m_job->parameters.depth);
+	       getFrameModeString(m_job->parameters.format).toLocal8Bit().data(),
+	       m_job->parameters.last_frame ? "SANE_TRUE" : "SANE_FALSE",
+	       m_job->parameters.bytes_per_line,
+	       m_job->parameters.pixels_per_line,
+	       m_job->parameters.lines,
+	       m_job->parameters.depth);
 
 	m_job->height = qMax(1, m_job->parameters.lines);
 	m_job->rowstride = m_job->parameters.pixels_per_line * 3; // Buffer is for a 24 bit RGB image
@@ -428,8 +416,7 @@ void ScannerSane::doGetParameters()
 	setState(State::READ);
 }
 
-void ScannerSane::doRead()
-{
+void ScannerSane::doRead() {
 	/* Read as many bytes as we need to complete a line */
 	int n_to_read = m_job->lineBuffer.size() - m_job->nUsed;
 
@@ -439,14 +426,14 @@ void ScannerSane::doRead()
 	qDebug("sane_read(%d) -> (%s, %d)", n_to_read, sane_strstatus(status), n_read);
 
 	/* Completed read */
-	if(status == SANE_STATUS_EOF){
+	if(status == SANE_STATUS_EOF) {
 		if(m_job->parameters.lines > 0 && m_job->lineCount != m_job->parameters.lines)
 			qWarning("Scan completed with %d lines, expected %d lines", m_job->lineCount, m_job->parameters.lines);
 		if(m_job->nUsed > 0)
 			qWarning("Scan complete with %d bytes of unused data", m_job->nUsed);
 		doCompletePage();
 		return;
-	}else if(status != SANE_STATUS_GOOD){
+	} else if(status != SANE_STATUS_GOOD) {
 		qWarning("Unable to read frame from device: %s", sane_strstatus(status));
 		failScan(_("Error communicating with scanner"));
 		return;
@@ -455,34 +442,34 @@ void ScannerSane::doRead()
 	m_job->nUsed += n_read;
 
 	/* If we completed a line, feed it out */
-	if(m_job->nUsed >= m_job->lineBuffer.size()){
+	if(m_job->nUsed >= m_job->lineBuffer.size()) {
 		Q_ASSERT(m_job->nUsed == m_job->parameters.bytes_per_line);
 		++m_job->lineCount;
 
 		// Resize image if necessary
-		if(m_job->lineCount > m_job->height){
+		if(m_job->lineCount > m_job->height) {
 			m_job->height = m_job->lineCount;
 			m_job->imgbuf.resize(m_job->rowstride * m_job->height);
 		}
 
 		// Write data
 		int offset = (m_job->lineCount - 1) * m_job->rowstride;
-		if(m_job->parameters.format == SANE_FRAME_GRAY){
-			for(int i = 0; i < m_job->lineBuffer.size(); ++i){
+		if(m_job->parameters.format == SANE_FRAME_GRAY) {
+			for(int i = 0; i < m_job->lineBuffer.size(); ++i) {
 				std::memset(&m_job->imgbuf[offset + 3*i], m_job->lineBuffer[i], 3);
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_RGB){
+		} else if(m_job->parameters.format == SANE_FRAME_RGB) {
 			std::memcpy(&m_job->imgbuf[offset], &m_job->lineBuffer[0], m_job->lineBuffer.size());
-		}else if(m_job->parameters.format == SANE_FRAME_RED){
-			for(int i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_RED) {
+			for(int i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 0] = m_job->lineBuffer[i];
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_GREEN){
-			for(int i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_GREEN) {
+			for(int i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 1] = m_job->lineBuffer[i];
 			}
-		}else if(m_job->parameters.format == SANE_FRAME_BLUE){
-			for(int i = 0; i < m_job->lineBuffer.size(); ++i){
+		} else if(m_job->parameters.format == SANE_FRAME_BLUE) {
+			for(int i = 0; i < m_job->lineBuffer.size(); ++i) {
 				m_job->imgbuf[offset + 3*i + 2] = m_job->lineBuffer[i];
 			}
 		}
@@ -493,25 +480,24 @@ void ScannerSane::doRead()
 	}
 }
 
-void ScannerSane::doCompletePage()
-{
-	if(!m_job->parameters.last_frame){
+void ScannerSane::doCompletePage() {
+	if(!m_job->parameters.last_frame) {
 		setState(State::GET_PARAMETERS);
 		return;
 	}
 	QString filename = m_job->params.filename;
-	if(m_job->params.type != ScanType::SINGLE){
+	if(m_job->params.type != ScanType::SINGLE) {
 		QFileInfo finfo(filename);
 		filename = QString("%1/%2_%3.%4").arg(finfo.absolutePath()).arg(finfo.completeBaseName()).arg(m_job->pageNumber).arg(finfo.completeSuffix());
 	}
-	if(!QImage(m_job->imgbuf.data(), m_job->rowstride / 3, m_job->height, m_job->rowstride, QImage::Format_RGB888).save(filename)){
+	if(!QImage(m_job->imgbuf.data(), m_job->rowstride / 3, m_job->height, m_job->rowstride, QImage::Format_RGB888).save(filename)) {
 		failScan(_("Failed to save image"));
 		return;
 	}
 	m_job->imgbuf.clear();
 	emit pageAvailable(filename);
 
-	if(m_job->params.type != ScanType::SINGLE){
+	if(m_job->params.type != ScanType::SINGLE) {
 		++m_job->pageNumber;
 		setState(State::GET_PARAMETERS);
 		return;
@@ -520,10 +506,9 @@ void ScannerSane::doCompletePage()
 	doStop();
 }
 
-void ScannerSane::doStop()
-{
-	if(m_job != nullptr){
-		if(m_job->handle != nullptr){
+void ScannerSane::doStop() {
+	if(m_job != nullptr) {
+		if(m_job->handle != nullptr) {
 			qDebug("sane_close()");
 			sane_close(m_job->handle);
 		}
@@ -534,10 +519,9 @@ void ScannerSane::doStop()
 }
 
 /***************************** Sane option stuff *****************************/
-const SANE_Option_Descriptor* ScannerSane::getOptionByName(const QMap<QString, int>& options, SANE_Handle, const QString& name, int &index)
-{
+const SANE_Option_Descriptor* ScannerSane::getOptionByName(const QMap<QString, int>& options, SANE_Handle, const QString& name, int &index) {
 	QMap<QString, int>::const_iterator it = options.find(name);
-	if(it != options.end()){
+	if(it != options.end()) {
 		index = it.value();
 		return sane_get_option_descriptor(m_job->handle, index);
 	}
@@ -545,8 +529,7 @@ const SANE_Option_Descriptor* ScannerSane::getOptionByName(const QMap<QString, i
 	return nullptr;
 }
 
-bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index)
-{
+bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index) {
 	/* Check if supports automatic option */
 	if((option->cap & SANE_CAP_AUTOMATIC) == 0)
 		return false;
@@ -558,9 +541,8 @@ bool ScannerSane::setDefaultOption(SANE_Handle handle, const SANE_Option_Descrip
 	return status == SANE_STATUS_GOOD;
 }
 
-void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, bool value, bool* result)
-{
-	if(option->type != SANE_TYPE_BOOL){
+void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, bool value, bool* result) {
+	if(option->type != SANE_TYPE_BOOL) {
 		return;
 	}
 
@@ -570,26 +552,25 @@ void ScannerSane::setBoolOption(SANE_Handle handle, const SANE_Option_Descriptor
 	qDebug("sane_control_option(%d, SANE_ACTION_SET_VALUE, %s) -> (%s, %s)", option_index, value ? "SANE_TRUE" : "SANE_FALSE", sane_strstatus(status), result ? "SANE_TRUE" : "SANE_FALSE");
 }
 
-void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, int value, int* result)
-{
-	if(option->type != SANE_TYPE_INT){
+void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor* option, SANE_Int option_index, int value, int* result) {
+	if(option->type != SANE_TYPE_INT) {
 		return;
 	}
 
 	SANE_Word v = static_cast<SANE_Word>(value);
-	if(option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		if(option->constraint.range->quant != 0)
 			v *= option->constraint.range->quant;
 		v = qMax(option->constraint.range->min, qMin(option->constraint.range->max, v));
-	}else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST){
+	} else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
 		int distance = std::numeric_limits<int>::max();
 		int nearest = 0;
 
 		/* Find nearest value to requested */
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			int x = option->constraint.word_list[i+1];
 			int d = qAbs(x - v);
-			if(d < distance){
+			if(d < distance) {
 				distance = d;
 				nearest = x;
 			}
@@ -602,26 +583,25 @@ void ScannerSane::setIntOption(SANE_Handle handle, const SANE_Option_Descriptor*
 	if(result) *result = v;
 }
 
-void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, double value, double* result)
-{
-	if(option->type != SANE_TYPE_FIXED){
+void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, double value, double* result) {
+	if(option->type != SANE_TYPE_FIXED) {
 		return;
 	}
 
 	double v = value;
-	if(option->constraint_type == SANE_CONSTRAINT_RANGE){
+	if(option->constraint_type == SANE_CONSTRAINT_RANGE) {
 		double min = SANE_UNFIX(option->constraint.range->min);
 		double max = SANE_UNFIX(option->constraint.range->max);
 		v = qMax(min, qMin(max, v));
-	}else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST){
+	} else if(option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
 		double distance = std::numeric_limits<double>::max();
 		double nearest = 0.0;
 
 		/* Find nearest value to requested */
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			double x = SANE_UNFIX(option->constraint.word_list[i+1]);
 			double d = qAbs(x - v);
-			if(d < distance){
+			if(d < distance) {
 				distance = d;
 				nearest = x;
 			}
@@ -636,9 +616,8 @@ void ScannerSane::setFixedOption(SANE_Handle handle, const SANE_Option_Descripto
 	if(result) *result = SANE_UNFIX(v_fixed);
 }
 
-bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const QString& value, QString* result)
-{
-	if(option->type != SANE_TYPE_STRING){
+bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const QString& value, QString* result) {
+	if(option->type != SANE_TYPE_STRING) {
 		return false;
 	}
 
@@ -652,18 +631,17 @@ bool ScannerSane::setStringOption(SANE_Handle handle, const SANE_Option_Descript
 	return status == SANE_STATUS_GOOD;
 }
 
-bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const QStringList& values, QString* result)
-{
-	if(option->type != SANE_TYPE_STRING){
+bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const QStringList& values, QString* result) {
+	if(option->type != SANE_TYPE_STRING) {
 		return false;
 	}
-	if(option->constraint_type != SANE_CONSTRAINT_STRING_LIST){
+	if(option->constraint_type != SANE_CONSTRAINT_STRING_LIST) {
 		return false;
 	}
 
-	for(const QString& value : values){
-		for(int j = 0; option->constraint.string_list[j] != nullptr; ++j){
-			if(value == option->constraint.string_list[j]){
+	for(const QString& value : values) {
+		for(int j = 0; option->constraint.string_list[j] != nullptr; ++j) {
+			if(value == option->constraint.string_list[j]) {
 				return setStringOption(handle, option, option_index, value, result);
 			}
 		}
@@ -672,8 +650,7 @@ bool ScannerSane::setConstrainedStringOption(SANE_Handle handle, const SANE_Opti
 	return false;
 }
 
-void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option)
-{
+void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option) {
 	QString s = QString("Option %1:").arg(index);
 
 	if(option->name && std::strlen(option->name) > 0)
@@ -683,22 +660,22 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		s += QString(" title='%1'").arg(option->title);
 
 	QString typestr[] = {"bool", "int", "fixed", "string", "button", "group"};
-	if(option->type <= SANE_TYPE_GROUP){
+	if(option->type <= SANE_TYPE_GROUP) {
 		s += QString(" type=%1").arg(typestr[option->type]);
-	}else{
+	} else {
 		s += QString(" type=%1").arg(option->type);
 	}
 
 	s += QString(" size=%1").arg(option->size);
 
 	QString unitstr[] = {"none", "pixel", "bit", "mm", "dpi", "percent", "microseconds"};
-	if(option->unit <= SANE_UNIT_MICROSECOND){
+	if(option->unit <= SANE_UNIT_MICROSECOND) {
 		s += QString(" unit=%1").arg(unitstr[option->unit]);
-	}else{
+	} else {
 		s += QString(" unit=%1").arg(option->unit);
 	}
 
-	switch(option->constraint_type){
+	switch(option->constraint_type) {
 	case SANE_CONSTRAINT_RANGE:
 		if(option->type == SANE_TYPE_FIXED)
 			s += QString(" min=%1, max=%2, quant=%3").arg(SANE_UNFIX(option->constraint.range->min)).arg(SANE_UNFIX(option->constraint.range->max)).arg(option->constraint.range->quant);
@@ -707,7 +684,7 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		break;
 	case SANE_CONSTRAINT_WORD_LIST:
 		s += " values=[";
-		for(int i = 0; i < option->constraint.word_list[0]; ++i){
+		for(int i = 0; i < option->constraint.word_list[0]; ++i) {
 			if(i > 0)
 				s += ", ";
 			if(option->type == SANE_TYPE_INT)
@@ -719,7 +696,7 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		break;
 	case SANE_CONSTRAINT_STRING_LIST:
 		s += " values=[";
-		for(int i = 0; option->constraint.string_list[i] != nullptr; ++i){
+		for(int i = 0; option->constraint.string_list[i] != nullptr; ++i) {
 			if(i > 0)
 				s += ", ";
 			s += QString("\"%1\"").arg(option->constraint.string_list[i]);
@@ -731,38 +708,38 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 	}
 
 	int cap = option->cap;
-	if(cap != 0){
+	if(cap != 0) {
 		s += " cap=";
-		if((cap & SANE_CAP_SOFT_SELECT) != 0){
+		if((cap & SANE_CAP_SOFT_SELECT) != 0) {
 			s += "soft-select,";
 			cap &= ~SANE_CAP_SOFT_SELECT;
 		}
-		if((cap & SANE_CAP_HARD_SELECT) != 0){
+		if((cap & SANE_CAP_HARD_SELECT) != 0) {
 			s += "hard-select,";
 			cap &= ~SANE_CAP_HARD_SELECT;
 		}
-		if((cap & SANE_CAP_SOFT_DETECT) != 0){
+		if((cap & SANE_CAP_SOFT_DETECT) != 0) {
 			s += "soft-detect,";
 			cap &= ~SANE_CAP_SOFT_DETECT;
 		}
-		if((cap & SANE_CAP_EMULATED) != 0){
+		if((cap & SANE_CAP_EMULATED) != 0) {
 			s += "emulated,";
 			cap &= ~SANE_CAP_EMULATED;
 		}
-		if((cap & SANE_CAP_AUTOMATIC) != 0){
+		if((cap & SANE_CAP_AUTOMATIC) != 0) {
 			s += "automatic,";
 			cap &= ~SANE_CAP_AUTOMATIC;
 		}
-		if((cap & SANE_CAP_INACTIVE) != 0){
+		if((cap & SANE_CAP_INACTIVE) != 0) {
 			s += "inactive,";
 			cap &= ~SANE_CAP_INACTIVE;
 		}
-		if((cap & SANE_CAP_ADVANCED) != 0){
+		if((cap & SANE_CAP_ADVANCED) != 0) {
 			s += "advanced,";
 			cap &= ~SANE_CAP_ADVANCED;
 		}
 		/* Unknown capabilities */
-		if(cap != 0){
+		if(cap != 0) {
 			s += QString("unknown (%1)").arg(cap);
 		}
 	}
@@ -772,12 +749,11 @@ void ScannerSane::logOption(SANE_Int index, const SANE_Option_Descriptor *option
 		qDebug("  Description: %s", option->desc);
 }
 
-QString ScannerSane::getFrameModeString(SANE_Frame frame)
-{
+QString ScannerSane::getFrameModeString(SANE_Frame frame) {
 	QString framestr[] = {"SANE_FRAME_GRAY", "SANE_FRAME_RGB", "SANE_FRAME_RED", "SANE_FRAME_GREEN", "SANE_FRAME_BLUE"};
-	if(frame <= SANE_FRAME_BLUE){
+	if(frame <= SANE_FRAME_BLUE) {
 		return framestr[frame];
-	}else{
+	} else {
 		return QString("SANE_FRAME(%1)").arg(frame);
 	}
 }
